@@ -6,7 +6,7 @@
  *
  * File begun on 2007-12-20 by RGerhards (extracted from syslogd.c)
  *
- * Copyright 2007-2016 Rainer Gerhards and Adiscon GmbH.
+ * Copyright 2007-2017 Rainer Gerhards and Adiscon GmbH.
  *
  * This file is part of rsyslog.
  *
@@ -58,7 +58,9 @@
 #include "hashtable.h"
 #include "ratelimit.h"
 
+#if !defined(_AIX)
 #pragma GCC diagnostic ignored "-Wswitch-enum"
+#endif
 
 MODULE_TYPE_INPUT
 MODULE_TYPE_NOKEEP
@@ -83,6 +85,9 @@ MODULE_CNFNAME("imuxsock")
 /* forward definitions */
 static rsRetVal resetConfigVariables(uchar __attribute__((unused)) *pp, void __attribute__((unused)) *pVal);
 
+#if defined(_AIX)
+#define ucred  ucred_t
+#endif 
 /* emulate struct ucred for platforms that do not have it */
 #ifndef HAVE_SCM_CREDENTIALS
 struct ucred { int pid; uid_t uid; gid_t gid; };
@@ -423,7 +428,8 @@ addListner(instanceConf_t *inst)
 	listeners[nfd].flags = inst->bIgnoreTimestamp ? IGNDATE : NOFLAG;
 	listeners[nfd].bCreatePath = inst->bCreatePath;
 	listeners[nfd].sockName = ustrdup(inst->sockName);
-	listeners[nfd].bUseCreds = (inst->bDiscardOwnMsgs || inst->bWritePid || inst->ratelimitInterval || inst->bAnnotate || inst->bUseSysTimeStamp) ? 1 : 0;
+	listeners[nfd].bUseCreds = (inst->bDiscardOwnMsgs || inst->bWritePid || inst->ratelimitInterval
+	|| inst->bAnnotate || inst->bUseSysTimeStamp) ? 1 : 0;
 	listeners[nfd].bAnnotate = inst->bAnnotate;
 	listeners[nfd].bParseTrusted = inst->bParseTrusted;
 	listeners[nfd].bDiscardOwnMsgs = inst->bDiscardOwnMsgs;
@@ -764,7 +770,7 @@ copyescaped(uchar *dstbuf, uchar *inbuf, int inlen)
 static rsRetVal
 SubmitMsg(uchar *pRcv, int lenRcv, lstn_t *pLstn, struct ucred *cred, struct timeval *ts)
 {
-	msg_t *pMsg = NULL;
+	smsg_t *pMsg = NULL;
 	int lenMsg;
 	int offs;
 	int i;
@@ -937,16 +943,18 @@ SubmitMsg(uchar *pRcv, int lenRcv, lstn_t *pLstn, struct ucred *cred, struct tim
 				}
 			} else {
 				if(datetime.ParseTIMESTAMP3339(&(pMsg->tTIMESTAMP), &parse, &lenMsg) != RS_RET_OK &&
-				   datetime.ParseTIMESTAMP3164(&(pMsg->tTIMESTAMP), &parse, &lenMsg, NO_PARSE3164_TZSTRING, NO_PERMIT_YEAR_AFTER_TIME) != RS_RET_OK) {
+				datetime.ParseTIMESTAMP3164(&(pMsg->tTIMESTAMP), &parse, &lenMsg,
+				NO_PARSE3164_TZSTRING, NO_PERMIT_YEAR_AFTER_TIME) != RS_RET_OK) {
 					DBGPRINTF("we have a problem, invalid timestamp in msg!\n");
 				}
 			}
 		} else { /* if we pulled the time from the system, we need to update the message text */
 			uchar *tmpParse = parse; /* just to check correctness of TS */
 			if(datetime.ParseTIMESTAMP3339(&dummyTS, &tmpParse, &lenMsg) == RS_RET_OK ||
-			   datetime.ParseTIMESTAMP3164(&dummyTS, &tmpParse, &lenMsg, NO_PARSE3164_TZSTRING, NO_PERMIT_YEAR_AFTER_TIME) == RS_RET_OK) {
-				/* We modify the message only if it contained a valid timestamp,
-				 * otherwise we do not touch it at all. */
+		 	datetime.ParseTIMESTAMP3164(&dummyTS, &tmpParse, &lenMsg, NO_PARSE3164_TZSTRING,
+			NO_PERMIT_YEAR_AFTER_TIME) == RS_RET_OK) {
+			/* We modify the message only if it contained a valid timestamp,
+			otherwise we do not touch it at all. */
 				datetime.formatTimestamp3164(&st, (char*)parse, 0);
 				parse[15] = ' '; /* re-write \0 from fromatTimestamp3164 by SP */
 				/* update "counters" to reflect processed timestamp */
@@ -991,8 +999,10 @@ finalize_it:
  * of the socket which is to be processed. This eases access to the
  * growing number of properties. -- rgerhards, 2008-08-01
  */
+#if !defined(_AIX)
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wcast-align" /* TODO: how can we fix these warnings? */
+#endif
 /* Problem with the warnings: they seem to stem back from the way the API is structured */
 static rsRetVal readSocket(lstn_t *pLstn)
 {
@@ -1038,9 +1048,13 @@ static rsRetVal readSocket(lstn_t *pLstn)
 	msgiov.iov_len = iMaxLine;
 	msgh.msg_iov = &msgiov;
 	msgh.msg_iovlen = 1;
+/*  AIXPORT : MSG_DONTWAIT not supported */
+#if defined (_AIX)
+#define MSG_DONTWAIT    MSG_NONBLOCK
+#endif
 	iRcvd = recvmsg(pLstn->fd, &msgh, MSG_DONTWAIT);
  
-	DBGPRINTF("Message from UNIX socket: #%d\n", pLstn->fd);
+	DBGPRINTF("Message from UNIX socket: #%d, size %d\n", pLstn->fd, (int) iRcvd);
 	if(iRcvd > 0) {
 		cred = NULL;
 		ts = NULL;
@@ -1077,7 +1091,9 @@ finalize_it:
 
 	RETiRet;
 }
+#if  !defined(_AIX)
 #pragma GCC diagnostic pop
+#endif
 
 
 /* activate current listeners */
@@ -1118,7 +1134,9 @@ activateListeners(void)
 		listeners[0].ratelimitInterval = runModConf->ratelimitIntervalSysSock;
 		listeners[0].ratelimitBurst = runModConf->ratelimitBurstSysSock;
 		listeners[0].ratelimitSev = runModConf->ratelimitSeveritySysSock;
-		listeners[0].bUseCreds = (runModConf->bWritePidSysSock || runModConf->ratelimitIntervalSysSock || runModConf->bAnnotateSysSock || runModConf->bDiscardOwnMsgs || runModConf->bUseSysTimeStamp) ? 1 : 0;
+		listeners[0].bUseCreds = (runModConf->bWritePidSysSock || runModConf->ratelimitIntervalSysSock
+		|| runModConf->bAnnotateSysSock || runModConf->bDiscardOwnMsgs
+		|| runModConf->bUseSysTimeStamp) ? 1 : 0;
 		listeners[0].bWritePid = runModConf->bWritePidSysSock;
 		listeners[0].bAnnotate = runModConf->bAnnotateSysSock;
 		listeners[0].bParseTrusted = runModConf->bParseTrusted;
@@ -1507,6 +1525,11 @@ BEGINafterRun
 	int i;
 CODESTARTafterRun
 	/* do cleanup here */
+        if(startIndexUxLocalSockets == 1 && nfd == 1) {
+                /* No sockets were configured, no cleanup needed. */
+                return RS_RET_OK;
+        }
+
 	/* Close the UNIX sockets. */
        for (i = 0; i < nfd; i++)
 		if (listeners[i].fd != -1)
