@@ -38,9 +38,7 @@
 #include <errno.h>
 #include <ctype.h>
 #include <unistd.h>
-#ifdef USE_NETZIP
 #include <zlib.h>
-#endif
 #include <pthread.h>
 #include <gssapi/gssapi.h>
 #include "dirty.h"
@@ -424,7 +422,6 @@ CODESTARTdoAction
 		if((int) l > iMaxLine)
 			l = iMaxLine;
 
-#		ifdef	USE_NETZIP
 		/* Check if we should compress and, if so, do it. We also
 		 * check if the message is large enough to justify compression.
 		 * The smaller the message, the less likely is a gain in compression.
@@ -466,7 +463,6 @@ CODESTARTdoAction
 			}
 			++destLen;
 		}
-#		endif
 
 		CHKiRet_Hdlr(tcpclt.Send(pData->pTCPClt, pData, psz, l)) {
 			/* error! */
@@ -477,12 +473,10 @@ CODESTARTdoAction
 		break;
 	}
 finalize_it:
-#	ifdef USE_NETZIP
 	if((psz != NULL) && (psz != (char*) ppString[0]))  {
 		/* we need to free temporary buffer, alloced above - Naoya Nakazawa, 2010-01-11 */
 		free(psz);
 	}
-#	endif
 	pthread_mutex_unlock(&mutDoAct);
 ENDdoAction
 
@@ -535,7 +529,6 @@ CODE_STD_STRING_REQUESTparseSelectorAct(1)
 			++p; /* eat '(' or ',' (depending on when called) */
 			/* check options */
 			if(*p == 'z') { /* compression */
-#					ifdef USE_NETZIP
 				++p; /* eat */
 				if(isdigit((int) *p)) {
 					int iLevel;
@@ -547,10 +540,6 @@ CODE_STD_STRING_REQUESTparseSelectorAct(1)
 						 "forwardig action - NOT turning on compression.",
 						 *p);
 				}
-#					else
-				errmsg.LogError(0, NO_ERRCODE, "Compression requested, but rsyslogd is not compiled "
-					 "with compression support - request ignored.");
-#					endif /* #ifdef USE_NETZIP */
 			} else if(*p == 'o') { /* octet-couting based TCP framing? */
 				++p; /* eat */
 				/* no further options settable */
@@ -720,10 +709,14 @@ CODEmodInit_QueryRegCFSLineHdlr
 	CHKiRet(objUse(gssutil, LM_GSSUTIL_FILENAME));
 	CHKiRet(objUse(tcpclt, LM_TCPCLT_FILENAME));
 
-	CHKiRet(omsdRegCFSLineHdlr((uchar *)"gssforwardservicename", 0, eCmdHdlrGetWord, NULL, &cs.gss_base_service_name, STD_LOADABLE_MODULE_ID));
-	CHKiRet(omsdRegCFSLineHdlr((uchar *)"gssmode", 0, eCmdHdlrGetWord, setGSSMode, &cs.gss_mode, STD_LOADABLE_MODULE_ID));
-	CHKiRet(omsdRegCFSLineHdlr((uchar *)"actiongssforwarddefaulttemplate", 0, eCmdHdlrGetWord, NULL, &cs.pszTplName, STD_LOADABLE_MODULE_ID));
-	CHKiRet(omsdRegCFSLineHdlr((uchar *)"resetconfigvariables", 1, eCmdHdlrCustomHandler, resetConfigVariables, NULL, STD_LOADABLE_MODULE_ID));
+	CHKiRet(omsdRegCFSLineHdlr((uchar *)"gssforwardservicename", 0, eCmdHdlrGetWord, NULL,
+	&cs.gss_base_service_name, STD_LOADABLE_MODULE_ID));
+	CHKiRet(omsdRegCFSLineHdlr((uchar *)"gssmode", 0, eCmdHdlrGetWord, setGSSMode, &cs.gss_mode,
+	STD_LOADABLE_MODULE_ID));
+	CHKiRet(omsdRegCFSLineHdlr((uchar *)"actiongssforwarddefaulttemplate", 0, eCmdHdlrGetWord, NULL,
+	&cs.pszTplName, STD_LOADABLE_MODULE_ID));
+	CHKiRet(omsdRegCFSLineHdlr((uchar *)"resetconfigvariables", 1, eCmdHdlrCustomHandler, resetConfigVariables,
+	NULL, STD_LOADABLE_MODULE_ID));
 ENDmodInit
 
 #endif /* #ifdef USE_GSSAPI */
