@@ -96,7 +96,7 @@ doLogMsg(const int iErrno, const int iErrCode,  const int severity, const char *
 			snprintf(buf, sizeof(buf), "%s [v%s try http://www.rsyslog.com/e/%d ]", msg, VERSION, iErrCode * -1);
 		}
 	}
-	buf[sizeof(buf)/sizeof(char) - 1] = '\0'; /* just to be on the safe side... */
+	buf[sizeof(buf) - 1] = '\0'; /* just to be on the safe side... */
 	errno = 0;
 	
 	glblErrLogger(severity, iErrCode, (uchar*)buf);
@@ -115,7 +115,7 @@ doLogMsg(const int iErrno, const int iErrCode,  const int severity, const char *
  * maps to a specific error event).
  * rgerhards, 2008-06-27
  */
-static void __attribute__((format(printf, 3, 4)))
+void __attribute__((format(printf, 3, 4)))
 LogError(const int iErrno, const int iErrCode, const char *fmt, ... )
 {
 	va_list ap;
@@ -129,7 +129,7 @@ LogError(const int iErrno, const int iErrCode, const char *fmt, ... )
 		lenBuf--;
 	}
 	va_end(ap);
-	buf[sizeof(buf)/sizeof(char) - 1] = '\0'; /* just to be on the safe side... */
+	buf[sizeof(buf) - 1] = '\0'; /* just to be on the safe side... */
 	
 	doLogMsg(iErrno, iErrCode, LOG_ERR, buf);
 }
@@ -144,7 +144,7 @@ LogError(const int iErrno, const int iErrCode, const char *fmt, ... )
  * maps to a specific error event).
  * rgerhards, 2008-06-27
  */
-static void __attribute__((format(printf, 4, 5)))
+void __attribute__((format(printf, 4, 5)))
 LogMsg(const int iErrno, const int iErrCode, const int severity, const char *fmt, ... )
 {
 	va_list ap;
@@ -158,7 +158,7 @@ LogMsg(const int iErrno, const int iErrCode, const int severity, const char *fmt
 		lenBuf--;
 	}
 	va_end(ap);
-	buf[sizeof(buf)/sizeof(char) - 1] = '\0'; /* just to be on the safe side... */
+	buf[sizeof(buf) - 1] = '\0'; /* just to be on the safe side... */
 	
 	doLogMsg(iErrno, iErrCode, severity, buf);
 }
