@@ -48,6 +48,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
+#include <stdarg.h>
 #define MAXFNAME 1024
 
 #include <ksi/ksi.h>
@@ -119,6 +120,8 @@ RSKSIE2String(int err)
 		return "unexpected end of log";
 	case RSGTE_EXTRACT_HASH:
 		return "either record-hash, left-hash or right-hash was empty";
+	case RSGTE_MISS_KSISIG:
+		return "KSI signature missing";
 	default:
 		return "unknown error";
 	}
@@ -134,8 +137,6 @@ hashOutputLengthOctetsKSI(uint8_t hashID)
 		return 32;
 	case KSI_HASHALG_RIPEMD160: /** The RIPEMD-160 algorithm. */
 		return 20;
-	case KSI_HASHALG_SHA2_224: /** The SHA-224 algorithm. */
-		return 28;
 	case KSI_HASHALG_SHA2_384: /** The SHA-384 algorithm. */
 		return 48;
 	case KSI_HASHALG_SHA2_512: /** The SHA-512 algorithm. */
@@ -164,8 +165,6 @@ hashIdentifierKSI(KSI_HashAlgorithm hashID)
 		return 0x01;
 	case KSI_HASHALG_RIPEMD160: /** The RIPEMD-160 algorithm. */
 		return 0x02;
-	case KSI_HASHALG_SHA2_224: /** The SHA-224 algorithm. */
-		return 0x03;
 	case KSI_HASHALG_SHA2_384: /** The SHA-384 algorithm. */
 		return 0x04;
 	case KSI_HASHALG_SHA2_512: /** The SHA-512 algorithm. */
@@ -194,14 +193,10 @@ hashAlgNameKSI(uint8_t hashID)
 		return "SHA2-256";
 	case KSI_HASHALG_RIPEMD160:
 		return "RIPEMD-160";
-	case KSI_HASHALG_SHA2_224:
-		return "SHA2-224";
 	case KSI_HASHALG_SHA2_384:
 		return "SHA2-384";
 	case KSI_HASHALG_SHA2_512:
 		return "SHA2-512";
-	case KSI_HASHALG_SHA3_244:
-		return "SHA3-224";
 	case KSI_HASHALG_SHA3_256:
 		return "SHA3-256";
 	case KSI_HASHALG_SHA3_384:
@@ -223,8 +218,6 @@ hashID2AlgKSI(uint8_t hashID)
 		return KSI_HASHALG_SHA2_256;
 	case 0x02:
 		return KSI_HASHALG_RIPEMD160;
-	case 0x03:
-		return KSI_HASHALG_SHA2_224;
 	case 0x04:
 		return KSI_HASHALG_SHA2_384;
 	case 0x05:
@@ -243,6 +236,25 @@ hashID2AlgKSI(uint8_t hashID)
 		return 0xff;
 	}
 }
+
+static void __attribute__ ((format (printf, 2, 3)))
+report(rsksictx ctx, const char *errmsg, ...) {
+	char buf[1024];
+	va_list args;
+	va_start(args, errmsg);
+
+	int r = vsnprintf(buf, sizeof (buf), errmsg, args);
+	buf[sizeof(buf)-1] = '\0';
+
+	if(ctx->logFunc == NULL)
+		return;
+
+	if(r>0 && r<(int)sizeof(buf))
+		ctx->logFunc(ctx->usrptr, (uchar*)buf);
+	else
+		ctx->logFunc(ctx->usrptr, (uchar*)errmsg);
+}
+
 static void
 reportErr(rsksictx ctx, const char *const errmsg)
 {
@@ -270,6 +282,13 @@ rsksisetErrFunc(rsksictx ctx, void (*func)(void*, uchar *), void *usrptr)
 	ctx->errFunc = func;
 }
 
+void
+rsksisetLogFunc(rsksictx ctx, void (*func)(void*, uchar *), void *usrptr)
+{
+	ctx->usrptr = usrptr;
+	ctx->logFunc = func;
+}
+
 int
 rsksiIntoImprintFromKSI_DataHash(imprint_t* imp, ksifile ksi, KSI_DataHash *hash)
 {
@@ -646,9 +665,16 @@ writeStateFile(ksifile ksi)
 	struct rsksistatefile sf;
 
 	fd = open((char*)ksi->statefilename,
-		       O_WRONLY|O_CREAT|O_TRUNC|O_NOCTTY|O_CLOEXEC, 0600);
+		       O_WRONLY|O_CREAT|O_TRUNC|O_NOCTTY|O_CLOEXEC, ksi->ctx->fCreateMode);
 	if(fd == -1)
 		goto done;
+	if (ksi->ctx->fileUID != (uid_t) - 1 || ksi->ctx->fileGID != (gid_t) - 1) {
+		/* we need to set owner/group */
+		if (fchown(fd, ksi->ctx->fileUID, ksi->ctx->fileGID) != 0) {
+			report(ksi->ctx, "lmsig_ksi: chown for file '%s' failed: %s",
+				ksi->statefilename, strerror(errno));
+		}
+	}
 
 	memcpy(sf.hdr, "KSISTAT10", 9);
 	sf.hashID = hashIdentifierKSI(ksi->hashAlg);
@@ -682,20 +708,48 @@ static int
 tlvOpenKSI(ksifile ksi, const char *const hdr, unsigned lenHdr)
 {
 	int r = 0;
+	struct stat stat_st;
+
 	ksi->fd = open((char*)ksi->sigfilename,
 		       O_WRONLY|O_APPEND|O_NOCTTY|O_CLOEXEC, 0600);
 	if(ksi->fd == -1) {
 		/* looks like we need to create a new file */
 		ksi->fd = open((char*)ksi->sigfilename,
-			       O_WRONLY|O_CREAT|O_NOCTTY|O_CLOEXEC, 0600);
+			       O_WRONLY|O_CREAT|O_NOCTTY|O_CLOEXEC, ksi->ctx->fCreateMode);
 		if(ksi->fd == -1) {
 			r = RSGTE_IO;
 			goto done;
 		}
+
+		/* check and set uid/gid */
+		if (ksi->ctx->fileUID != (uid_t) - 1 || ksi->ctx->fileGID != (gid_t) - 1) {
+			/* we need to set owner/group */
+			if (fchown(ksi->fd, ksi->ctx->fileUID, ksi->ctx->fileGID) != 0) {
+				report(ksi->ctx, "lmsig_ksi: chown for file '%s' failed: %s",
+					ksi->sigfilename, strerror(errno));
+			}
+		}
+
+		/* Write fileHeader */
 		memcpy(ksi->tlvBuf, hdr, lenHdr);
 		ksi->tlvIdx = lenHdr;
 	} else {
-		ksi->tlvIdx = 0; /* header already present! */
+		/* Get FileSize from existing ksisigfile */
+		if(fstat(ksi->fd, &stat_st) == -1) {
+			reportErr(ksi->ctx, "tlvOpenKSI: can not stat file");
+			r = RSGTE_IO;
+			goto done;
+		}
+
+		/* Check if size is above header length. */
+		if(stat_st.st_size > 0) {
+			/* header already present! */
+			ksi->tlvIdx = 0;
+		} else {
+			/* Write fileHeader */
+			memcpy(ksi->tlvBuf, hdr, lenHdr);
+			ksi->tlvIdx = lenHdr;
+		}
 	}
 	/* we now need to obtain the last previous hash, so that
 	 * we can continue the hash chain. We do not check for error
@@ -742,6 +796,12 @@ rsksiCtxNew(void)
 	ctx->hashAlg = KSI_HASHALG_SHA2_256;
 	ctx->errFunc = NULL;
 	ctx->usrptr = NULL;
+	ctx->fileUID = -1;
+	ctx->fileGID = -1;
+	ctx->dirUID = -1;
+	ctx->dirGID = -1;
+	ctx->fCreateMode = 0644;
+	ctx->fDirCreateMode = 0700;
 	ctx->timestamper = strdup(
 			   "http://stamper.guardtime.net/gt-signingservice");
 	return ctx;
@@ -773,7 +833,9 @@ done:	return ksi;
 }
  
 
-/* returns 0 on succes, 1 if algo is unknown */
+/* returns 0 on succes, 1 if algo is unknown, 2 is algo has been remove
+ * because it is now considered insecure
+ */
 int
 rsksiSetHashFunction(rsksictx ctx, char *algName)
 {
@@ -784,8 +846,6 @@ rsksiSetHashFunction(rsksictx ctx, char *algName)
 		ctx->hashAlg = KSI_HASHALG_SHA2_256;
 	else if(!strcmp(algName, "RIPEMD-160"))
 		ctx->hashAlg = KSI_HASHALG_RIPEMD160;
-	else if(!strcmp(algName, "SHA2-224"))
-		ctx->hashAlg = KSI_HASHALG_SHA2_224;
 	else if(!strcmp(algName, "SHA2-384"))
 		ctx->hashAlg = KSI_HASHALG_SHA2_384;
 	else if(!strcmp(algName, "SHA2-512"))
@@ -800,6 +860,8 @@ rsksiSetHashFunction(rsksictx ctx, char *algName)
 		ctx->hashAlg = KSI_HASHALG_SHA3_512;
 	else if(!strcmp(algName, "SM3"))
 		ctx->hashAlg = KSI_HASHALG_SM3;
+	else if(!strcmp(algName, "SHA2-224"))
+		r = 2;
 	else
 		r = 1;
 	return r;
@@ -814,7 +876,6 @@ rsksifileDestruct(ksifile ksi)
 
 	if(!ksi->disabled && ksi->bInBlk) {
 		r = sigblkFinishKSI(ksi);
-		if(r != 0) ksi->disabled = 1;
 	}
 	if(!ksi->disabled)
 		r = tlvCloseKSI(ksi);
@@ -846,6 +907,10 @@ sigblkInitKSI(ksifile ksi)
 	ksi->nRoots = 0;
 	ksi->nRecords = 0;
 	ksi->bInBlk = 1;
+
+	report(ksi->ctx, "Started new block for signing, signature file %s, block count %lu", ksi->sigfilename,
+	ksi->blockSizeLimit);
+
 done:	return;
 }
 
@@ -1010,14 +1075,10 @@ sigblkAddRecordKSI(ksifile ksi, const uchar *rec, const size_t len)
 	KSI_DataHash_free(r);
 
 	if(ksi->nRecords == ksi->blockSizeLimit) {
-		ret = sigblkFinishKSI(ksi);
-		if(ret != 0) goto done;
+		sigblkFinishKSI(ksi);
 		sigblkInitKSI(ksi);
 	}
 done:	
-	if(ret != 0) {
-		ksi->disabled = 1;
-	}
 	return ret;
 }
 
@@ -1025,7 +1086,7 @@ static int
 signIt(ksifile ksi, KSI_DataHash *hash)
 {
 	unsigned char *der = NULL;
-	size_t lenDer;
+	size_t lenDer = 0;
 	int r = KSI_OK;
 	int ret = 0;
 	KSI_Signature *sig = NULL;
@@ -1055,7 +1116,15 @@ signIt(ksifile ksi, KSI_DataHash *hash)
 		goto done;
 	}
 
-	tlvWriteBlockSigKSI(ksi, der, lenDer);
+	r = tlvWriteBlockSigKSI(ksi, der, lenDer);
+	if(r != KSI_OK) {
+		reportKSIAPIErr(ksi->ctx, ksi, "tlvWriteBlockSigKSI", r);
+		ret = 1;
+		goto done;
+	}
+
+	report(ksi->ctx, "KSI signature appended to file %s, block count %lu", ksi->sigfilename, ksi->nRecords);
+
 done:
 	if (sig != NULL)
 		KSI_Signature_free(sig);
@@ -1089,7 +1158,7 @@ sigblkFinishKSI(ksifile ksi)
 			if(ret != 0) goto done; /* checks hash_node_ksi() result! */
 		}
 	}
-	if((ret = signIt(ksi, root)) != 0) goto done;
+	signIt(ksi, root);
 
 	KSI_DataHash_free(root);
 done:
