@@ -208,7 +208,8 @@ modUsrPrintAll(void)
 
 	BEGINfunc
 	for(pMod = pLoadedModules ; pMod != NULL ; pMod = pMod->pNext) {
-		dbgprintf("printing users of loadable module %s, refcount %u, ptr %p, type %d\n", pMod->pszName, pMod->uRefCnt, pMod, pMod->eType);
+		dbgprintf("printing users of loadable module %s, refcount %u, ptr %p, type %d\n",
+		pMod->pszName, pMod->uRefCnt, pMod, pMod->eType);
 		modUsrPrint(pMod);
 	}
 	ENDfunc
@@ -338,7 +339,7 @@ static uchar *modGetStateName(modInfo_t *pThis)
 
 /* Add a module to the loaded module linked list
  */
-static inline void
+static void
 addModToGlblList(modInfo_t *pThis)
 {
 	assert(pThis != NULL);
@@ -586,7 +587,7 @@ doModInit(rsRetVal (*modInit)(int, int*, rsRetVal(**)(), rsRetVal(*)(), modInfo_
 
 	if((iRet = moduleConstruct(&pNew)) != RS_RET_OK) {
 		pNew = NULL;
-		ABORT_FINALIZE(iRet);
+		FINALIZE;
 	}
 
 	CHKiRet((*modInit)(CURR_MOD_IF_VERSION, &pNew->iIFVers, &pNew->modQueryEtryPt, queryHostEtryPt, pNew));
@@ -687,7 +688,12 @@ doModInit(rsRetVal (*modInit)(int, int*, rsRetVal(**)(), rsRetVal(*)(), modInfo_
 			pNew->mod.om.supportsTX = 1;
 			localRet = (*pNew->modQueryEtryPt)((uchar*)"beginTransaction", &pNew->mod.om.beginTransaction);
 			if(localRet == RS_RET_MODULE_ENTRY_POINT_NOT_FOUND) {
+#ifdef _AIX
+/* AIXPORT : typecaste the return type for AIX */
+				pNew->mod.om.beginTransaction = (rsRetVal(*)(void*))dummyBeginTransaction;
+#else
 				pNew->mod.om.beginTransaction = dummyBeginTransaction;
+#endif
 				pNew->mod.om.supportsTX = 0;
 			} else if(localRet != RS_RET_OK) {
 				ABORT_FINALIZE(localRet);
@@ -738,7 +744,12 @@ doModInit(rsRetVal (*modInit)(int, int*, rsRetVal(**)(), rsRetVal(*)(), modInfo_
 			localRet = (*pNew->modQueryEtryPt)((uchar*)"endTransaction",
 				   &pNew->mod.om.endTransaction);
 			if(localRet == RS_RET_MODULE_ENTRY_POINT_NOT_FOUND) {
+#ifdef _AIX
+/* AIXPORT : typecaste the return type for AIX */
+				pNew->mod.om.endTransaction = (rsRetVal(*)(void*))dummyEndTransaction;
+#else
 				pNew->mod.om.endTransaction = dummyEndTransaction;
+#endif
 			} else if(localRet != RS_RET_OK) {
 				ABORT_FINALIZE(localRet);
 			}
@@ -895,10 +906,20 @@ static void modPrintList(void)
 								    NULL :  pMod->mod.om.newActInst);
 			dbgprintf("\ttryResume:          %p\n", pMod->tryResume);
 			dbgprintf("\tdoHUP:              %p\n", pMod->doHUP);
+#ifdef _AIX
+/* AIXPORT : typecaste the return type in AIX  */
+			dbgprintf("\tBeginTransaction:   %p\n",
+((pMod->mod.om.beginTransaction == (rsRetVal (*) (void*))dummyBeginTransaction) ?
+								   NULL :  pMod->mod.om.beginTransaction));
+			dbgprintf("\tEndTransaction:     %p\n",
+			((pMod->mod.om.endTransaction == (rsRetVal (*)(void*))dummyEndTransaction) ?
+								   NULL :  pMod->mod.om.endTransaction));
+#else
 			dbgprintf("\tBeginTransaction:   %p\n", ((pMod->mod.om.beginTransaction == dummyBeginTransaction) ?
 								   NULL :  pMod->mod.om.beginTransaction));
 			dbgprintf("\tEndTransaction:     %p\n", ((pMod->mod.om.endTransaction == dummyEndTransaction) ?
 								   NULL :  pMod->mod.om.endTransaction));
+#endif
 			break;
 		case eMOD_IN:
 			dbgprintf("Input Module Entry Points\n");
@@ -1024,7 +1045,7 @@ modUnloadAndDestructAll(eModLinkType_t modLinkTypesToUnload)
 }
 
 /* find module with given name in global list */
-static inline rsRetVal
+static rsRetVal
 findModule(uchar *pModName, int iModNameLen, modInfo_t **pMod)
 {
 	modInfo_t *pModInfo;
@@ -1229,7 +1250,13 @@ Load(uchar *pModName, sbool bConfLoad, struct nvlst *lst)
 		dlclose(pModHdlr);
 		ABORT_FINALIZE(RS_RET_MODULE_LOAD_ERR_NO_INIT);
 	}
+#ifdef _AIX
+/*  AIXPORT : typecaste address of pModInit  in AIX */
+	if((iRet = doModInit((rsRetVal(*)(int,int*,rsRetVal(**)(),rsRetVal(*)(),struct modInfo_s*))pModInit,
+(uchar*) pModName, pModHdlr, &pModInfo)) != RS_RET_OK) {
+#else
 	if((iRet = doModInit(pModInit, (uchar*) pModName, pModHdlr, &pModInfo)) != RS_RET_OK) {
+#endif
 		errmsg.LogError(0, RS_RET_MODULE_LOAD_ERR_INIT_FAILED,
 				"could not load module '%s', rsyslog error %d\n", pPathBuf, iRet);
 		dlclose(pModHdlr);
