@@ -687,7 +687,8 @@ static rsRetVal AddAllowedSender(struct AllowedSenders **ppRoot, struct AllowedS
 			        errmsg.LogError(0, NO_ERRCODE, "DNS error: Can't resolve \"%s\"", iAllow->addr.HostWildcard);
 				
 				if (ACLAddHostnameOnFail) {
-				        errmsg.LogError(0, NO_ERRCODE, "Adding hostname \"%s\" to ACL as a wildcard entry.", iAllow->addr.HostWildcard);
+				        errmsg.LogError(0, NO_ERRCODE, "Adding hostname \"%s\" to ACL as a wildcard "
+					"entry.", iAllow->addr.HostWildcard);
 				        iRet = AddAllowedSenderEntry(ppRoot, ppLast, iAllow, iSignificantBits);
 					FINALIZE;
 				} else {
@@ -782,10 +783,24 @@ PrintAllowedSenders(int iListToPrint)
 	struct AllowedSenders *pSender;
 	uchar szIP[64];
 	
-	assert((iListToPrint == 1) || (iListToPrint == 2)
-#ifdef USE_GSSAPI
-	       || (iListToPrint == 3)
-#endif
+#ifdef _AIX
+#ifdef USE_GSSAPI 
+	assert((iListToPrint == 1) || (iListToPrint == 2) || (iListToPrint == 3));
+	dbgprintf("Allowed %s Senders:\n",
+	       (iListToPrint == 1) ? "UDP" :
+	       (iListToPrint == 3) ? "GSS" :
+	       "TCP");
+#else 
+	assert((iListToPrint == 1) || (iListToPrint == 2));
+	dbgprintf("Allowed %s Senders:\n",
+	       (iListToPrint == 1) ? "UDP" :
+	       "TCP");
+#endif /* USE_GSSAPI */
+#else /* _AIX */
+	assert((iListToPrint == 1) || (iListToPrint == 2) 
+#ifdef USE_GSSAPI 
+	       || (iListToPrint == 3)  
+#endif 
 	       );
 
 	dbgprintf("Allowed %s Senders:\n",
@@ -794,6 +809,7 @@ PrintAllowedSenders(int iListToPrint)
 	       (iListToPrint == 3) ? "GSS" :
 #endif
 	       "TCP");
+#endif /* End of _AIX */
 
 	pSender = (iListToPrint == 1) ? pAllowedSenders_UDP :
 #ifdef USE_GSSAPI
@@ -1140,28 +1156,31 @@ cvthname(struct sockaddr_storage *f, prop_t **localName, prop_t **fqdn, prop_t *
  * normalized to lower case. The hostname is kept in mixed case for historic
  * reasons.
  */
+#define EMPTY_HOSTNAME_REPLACEMENT "localhost-empty-hostname"
 static rsRetVal
 getLocalHostname(uchar **ppName)
 {
 	DEFiRet;
 	char hnbuf[8192];
 	uchar *fqdn = NULL;
+	int empty_hostname = 1;
 
 	if(gethostname(hnbuf, sizeof(hnbuf)) != 0) {
-		strcpy(hnbuf, "localhost");
+		strcpy(hnbuf, EMPTY_HOSTNAME_REPLACEMENT);
 	} else {
 		/* now guard against empty hostname
 		 * see https://github.com/rsyslog/rsyslog/issues/1040
 		 */
 		if(hnbuf[0] == '\0') {
-			strcpy(hnbuf, "localhost");
+			strcpy(hnbuf, EMPTY_HOSTNAME_REPLACEMENT);
 		} else {
+			empty_hostname = 0;
 			hnbuf[sizeof(hnbuf)-1] = '\0'; /* be on the safe side... */
 		}
 	}
 
 	char *dot = strstr(hnbuf, ".");
-	if(dot == NULL) {
+	if(!empty_hostname && dot == NULL) {
 		/* we need to (try) to find the real name via resolver */
 		struct hostent *hent = gethostbyname((char*)hnbuf);
 		if(hent) {
@@ -1223,7 +1242,7 @@ closeUDPListenSockets(int *pSockArr)
  * param rcvbuf indicates desired rcvbuf size; 0 means OS default
  */
 static int *
-create_udp_socket(uchar *hostname, uchar *pszPort, int bIsServer, int rcvbuf, int ipfreebind)
+create_udp_socket(uchar *hostname, uchar *pszPort, int bIsServer, int rcvbuf, int ipfreebind, char *device)
 {
         struct addrinfo hints, *res, *r;
         int error, maxs, *s, *socks, on = 1;
@@ -1240,6 +1259,12 @@ create_udp_socket(uchar *hostname, uchar *pszPort, int bIsServer, int rcvbuf, in
 		hints.ai_flags = AI_NUMERICSERV;
         hints.ai_family = glbl.GetDefPFFamily();
         hints.ai_socktype = SOCK_DGRAM;
+ #if defined (_AIX)
+/* AIXPORT : SOCK_DGRAM has the protocol IPPROTO_UDP 
+ *           getaddrinfo needs this hint on AIX
+ */
+        hints.ai_protocol = IPPROTO_UDP;
+#endif
         error = getaddrinfo((char*) hostname, (char*) pszPort, &hints, &res);
         if(error) {
                errmsg.LogError(0, NO_ERRCODE, "%s",  gai_strerror(error));
@@ -1252,7 +1277,8 @@ create_udp_socket(uchar *hostname, uchar *pszPort, int bIsServer, int rcvbuf, in
 		/* EMPTY */;
         socks = MALLOC((maxs+1) * sizeof(int));
         if (socks == NULL) {
-               errmsg.LogError(0, NO_ERRCODE, "couldn't allocate memory for UDP sockets, suspending UDP message reception");
+		errmsg.LogError(0, NO_ERRCODE, "couldn't allocate memory for UDP sockets, suspending UDP "
+		"message reception");
                freeaddrinfo(res);
                return NULL;
         }
@@ -1260,7 +1286,12 @@ create_udp_socket(uchar *hostname, uchar *pszPort, int bIsServer, int rcvbuf, in
         *socks = 0;   /* num of sockets counter at start of array */
         s = socks + 1;
 	for (r = res; r != NULL ; r = r->ai_next) {
+#if defined (_AIX)
+/* AIXPORT : socktype will be SOCK_DGRAM, as set in hints above */
+               *s = socket(r->ai_family, SOCK_DGRAM, r->ai_protocol);
+#else
                *s = socket(r->ai_family, r->ai_socktype, r->ai_protocol);
+#endif
         	if (*s < 0) {
 			if(!(r->ai_family == PF_INET6 && errno == EAFNOSUPPORT))
 				errmsg.LogError(errno, NO_ERRCODE, "create_udp_socket(), socket");
@@ -1283,6 +1314,18 @@ create_udp_socket(uchar *hostname, uchar *pszPort, int bIsServer, int rcvbuf, in
                 }
 #		endif
 
+		if(device) {
+#			if defined(SO_BINDTODEVICE)
+			if(setsockopt(*s, SOL_SOCKET, SO_BINDTODEVICE, device, strlen(device) + 1) < 0)
+#			endif
+			{
+				errmsg.LogError(errno, NO_ERRCODE, "setsockopt(SO_BINDTODEVICE)");
+                                close(*s);
+				*s = -1;
+				continue;
+			}
+		}
+
 		/* if we have an error, we "just" suspend that socket. Eventually
 		 * other sockets will work. At the end of this function, we check
 		 * if we managed to open at least one socket. If not, we'll write
@@ -1301,7 +1344,10 @@ create_udp_socket(uchar *hostname, uchar *pszPort, int bIsServer, int rcvbuf, in
 		/* We need to enable BSD compatibility. Otherwise an attacker
 		 * could flood our log files by sending us tons of ICMP errors.
 		 */
-#if !defined(OS_BSD) && !defined(__hpux)
+/* AIXPORT : SO_BSDCOMPAT socket option is depricated , and its usage has been discontinued
+             on most unixes, AIX does not support this option , hence avoid the call.
+*/
+#if !defined(OS_BSD) && !defined(__hpux)  && !defined(_AIX)
 		if (should_use_so_bsdcompat()) {
 			if (setsockopt(*s, SOL_SOCKET, SO_BSDCOMPAT,
 					(char *) &on, sizeof(on)) < 0) {
@@ -1492,14 +1538,21 @@ finalize_it:
  * However, it caches entries in order to avoid too-frequent requery.
  * rgerhards, 2012-03-06
  */
+#if !defined(_AIX)
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wcast-align" /* TODO: how can we fix these warnings? */
 /* Problem with the warnings: they seem to stem back from the way the API is structured */
+#endif
 static rsRetVal
 getIFIPAddr(uchar *szif, int family, uchar *pszbuf, int lenBuf)
 {
+#ifdef _AIX
+	struct ifaddrs_rsys * ifaddrs = NULL;
+	struct ifaddrs_rsys * ifa;
+#else
 	struct ifaddrs * ifaddrs = NULL;
 	struct ifaddrs * ifa;
+#endif
 	void * pAddr;
 	DEFiRet;
 
@@ -1533,7 +1586,9 @@ finalize_it:
 	RETiRet;
 
 }
+#if !defined(_AIX)
 #pragma GCC diagnostic pop
+#endif
 
 
 /* queryInterface function
