@@ -309,7 +309,7 @@ ENDobjDestruct(parser)
  * pMsg->pszRawMsg buffer is updated.
  * rgerhards, 2008-10-09
  */
-static inline rsRetVal uncompressMessage(msg_t *pMsg)
+static rsRetVal uncompressMessage(smsg_t *pMsg)
 {
 	DEFiRet;
 	uchar *deflateBuf = NULL;
@@ -377,7 +377,7 @@ finalize_it:
  * rgerhards, 2007-09-14
  */
 static rsRetVal
-SanitizeMsg(msg_t *pMsg)
+SanitizeMsg(smsg_t *pMsg)
 {
 	DEFiRet;
 	uchar *pszMsg;
@@ -464,9 +464,15 @@ SanitizeMsg(msg_t *pMsg)
 	if(maxDest < sizeof(szSanBuf))
 		pDst = szSanBuf;
 	else 
-		CHKmalloc(pDst = MALLOC(iMaxLine + 1));
+		CHKmalloc(pDst = MALLOC(maxDest + 1));
 	if(iSrc > 0) {
 		iSrc--; /* go back to where everything is OK */
+		if(iSrc > maxDest) {
+			DBGPRINTF("parser.Sanitize: have oversize index %zd, "
+				"max %zd - corrected, but should not happen\n",
+				iSrc, maxDest);
+			iSrc = maxDest;
+		}
 		memcpy(pDst, pszMsg, iSrc); /* fast copy known good */
 	}
 	iDst = iSrc;
@@ -567,8 +573,8 @@ finalize_it:
  * this module as it is expected that allmost all parsers will need
  * that functionality and so they do not need to implement it themsleves.
  */
-static inline rsRetVal
-ParsePRI(msg_t *pMsg)
+static rsRetVal
+ParsePRI(smsg_t *pMsg)
 {
 	syslog_pri_t pri;
 	uchar *msg;
@@ -609,7 +615,7 @@ ParsePRI(msg_t *pMsg)
  * rgerhards, 2008-10-09
  */
 static rsRetVal
-ParseMsg(msg_t *pMsg)
+ParseMsg(smsg_t *pMsg)
 {
 	rsRetVal localRet = RS_RET_ERR;
 	parserList_t *pParserList;
