@@ -45,10 +45,10 @@ DEFobjCurrIf(parser)
 /* static data */
 
 /* generate a "repeated n times" message */
-static msg_t *
+static smsg_t *
 ratelimitGenRepMsg(ratelimit_t *ratelimit)
 {
-	msg_t *repMsg;
+	smsg_t *repMsg;
 	size_t lenRepMsg;
 	uchar szRepMsg[1024];
 
@@ -69,8 +69,8 @@ ratelimitGenRepMsg(ratelimit_t *ratelimit)
 done:	return repMsg;
 }
 
-static inline rsRetVal
-doLastMessageRepeatedNTimes(ratelimit_t *ratelimit, msg_t *pMsg, msg_t **ppRepMsg)
+static rsRetVal
+doLastMessageRepeatedNTimes(ratelimit_t *ratelimit, smsg_t *pMsg, smsg_t **ppRepMsg)
 {
 	int bNeedUnlockMutex = 0;
 	DEFiRet;
@@ -132,7 +132,7 @@ tellLostCnt(ratelimit_t *ratelimit)
  * be called concurrently.
  */
 static int
-withinRatelimit(ratelimit_t *ratelimit, time_t tt)
+withinRatelimit(ratelimit_t *ratelimit, time_t tt, char* appname)
 {
 	int ret;
 	uchar msgbuf[1024];
@@ -156,8 +156,8 @@ withinRatelimit(ratelimit_t *ratelimit, time_t tt)
 	if(ratelimit->begin == 0)
 		ratelimit->begin = tt;
 
-	/* resume if we go out of time window */
-	if(tt > ratelimit->begin + ratelimit->interval) {
+	/* resume if we go out of time window or if time has gone backwards */
+	if((tt > ratelimit->begin + ratelimit->interval) || (tt < ratelimit->begin) ) {
 		ratelimit->begin = 0;
 		ratelimit->done = 0;
 		tellLostCnt(ratelimit);
@@ -171,8 +171,8 @@ withinRatelimit(ratelimit_t *ratelimit, time_t tt)
 		ratelimit->missed++;
 		if(ratelimit->missed == 1) {
 			snprintf((char*)msgbuf, sizeof(msgbuf),
-			         "%s: begin to drop messages due to rate-limiting",
-				 ratelimit->name);
+                     "%s from <%s>: begin to drop messages due to rate-limiting",
+                 ratelimit->name, appname);
 			logmsgInternal(RS_RET_RATE_LIMITED, LOG_SYSLOG|LOG_INFO, msgbuf, 0);
 		}
 		ret = 0;
@@ -198,7 +198,7 @@ finalize_it:
  * message before the original message.
  */
 rsRetVal
-ratelimitMsg(ratelimit_t *ratelimit, msg_t *pMsg, msg_t **ppRepMsg)
+ratelimitMsg(ratelimit_t *ratelimit, smsg_t *pMsg, smsg_t **ppRepMsg)
 {
 	DEFiRet;
 	rsRetVal localRet;
@@ -215,7 +215,9 @@ ratelimitMsg(ratelimit_t *ratelimit, msg_t *pMsg, msg_t **ppRepMsg)
 	/* Only the messages having severity level at or below the
 	 * treshold (the value is >=) are subject to ratelimiting. */
 	if(ratelimit->interval && (pMsg->iSeverity >= ratelimit->severity)) {
-		if(withinRatelimit(ratelimit, pMsg->ttGenTime) == 0) {
+        char namebuf[512]; /* 256 for FGDN adn 256 for APPNAME should be enough */
+        snprintf(namebuf, sizeof namebuf, "%s%s", getHOSTNAME(pMsg), getAPPNAME(pMsg, 0));
+        if(withinRatelimit(ratelimit, pMsg->ttGenTime, namebuf) == 0) {
 			msgDestruct(&pMsg);
 			ABORT_FINALIZE(RS_RET_DISCARDMSG);
 		}
@@ -245,10 +247,10 @@ ratelimitChecked(ratelimit_t *ratelimit)
  * if pMultiSub == NULL, a single-message enqueue happens (under reconsideration)
  */
 rsRetVal
-ratelimitAddMsg(ratelimit_t *ratelimit, multi_submit_t *pMultiSub, msg_t *pMsg)
+ratelimitAddMsg(ratelimit_t *ratelimit, multi_submit_t *pMultiSub, smsg_t *pMsg)
 {
 	rsRetVal localRet;
-	msg_t *repMsg;
+	smsg_t *repMsg;
 	DEFiRet;
 
 	if(pMultiSub == NULL) {
@@ -353,7 +355,7 @@ ratelimitSetSeverity(ratelimit_t *ratelimit, intTiny severity)
 void
 ratelimitDestruct(ratelimit_t *ratelimit)
 {
-	msg_t *pMsg;
+	smsg_t *pMsg;
 	if(ratelimit->pMsg != NULL) {
 		if(ratelimit->nsupp > 0) {
 			pMsg = ratelimitGenRepMsg(ratelimit);
