@@ -27,6 +27,7 @@
 #define __SRUTILS_H_INCLUDED__ 1
 
 #include <stdlib.h>
+#include <stdint.h>
 
 /* syslog names */
 #ifndef LOG_MAKEPRI
@@ -78,11 +79,12 @@ unsigned char *srUtilStrDup(unsigned char *pOld, size_t len);
  * for it.
  * added 2007-07-17 by rgerhards
  */
-int makeFileParentDirs(const uchar *const szFile, size_t lenFile, mode_t mode, uid_t uid, gid_t gid, int bFailOnChown);
+int makeFileParentDirs(const uchar *const szFile, size_t lenFile, mode_t mode, uid_t uid, gid_t gid,
+int bFailOnChown);
 int execProg(uchar *program, int bWait, uchar *arg);
 void skipWhiteSpace(uchar **pp);
 rsRetVal genFileName(uchar **ppName, uchar *pDirName, size_t lenDirName, uchar *pFName,
-		     size_t lenFName, long lNum, int lNumDigits);
+		     size_t lenFName, int64_t lNum, int lNumDigits);
 int getNumberDigits(long lNum);
 rsRetVal timeoutComp(struct timespec *pt, long iTimeout);
 long timeoutVal(struct timespec *pt);
