@@ -41,8 +41,10 @@
 #include <assert.h>
 #include <sys/wait.h>
 #include <ctype.h>
+#include <inttypes.h>
 #include "srUtils.h"
 #include "obj.h"
+#include "errmsg.h"
 
 #if _POSIX_TIMERS <= 0
 #include <sys/time.h>
@@ -219,12 +221,18 @@ again:
                                 if(mkdir((char*)pszWork, mode) == 0) {
 					if(uid != (uid_t) -1 || gid != (gid_t) -1) {
 						/* we need to set owner/group */
-						if(chown((char*)pszWork, uid, gid) != 0)
+						if(chown((char*)pszWork, uid, gid) != 0) {
+							char errStr[1024]; /* buffer for strerr() */
+							rs_strerror_r(errno, errStr, sizeof(errStr));
+							LogError(0, RS_RET_DIR_CHOWN_ERROR,
+								"chown for directory '%s' failed: %s",
+								pszWork, errStr);
 							if(bFailOnChownFail)
 								bErr = 1;
 							/* silently ignore if configured
 							 * to do so.
 							 */
+						}
 					}
 				} else {
 					if(errno == EEXIST && iTry == 0) {
@@ -333,10 +341,12 @@ void skipWhiteSpace(uchar **pp)
  * to use as few space as possible.
  * rgerhards, 2008-01-03
  */
+#if !defined(_AIX)
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wformat-nonliteral"
+#endif
 rsRetVal genFileName(uchar **ppName, uchar *pDirName, size_t lenDirName, uchar *pFName,
-		     size_t lenFName, long lNum, int lNumDigits)
+		     size_t lenFName, int64_t lNum, int lNumDigits)
 {
 	DEFiRet;
 	uchar *pName;
@@ -351,10 +361,10 @@ rsRetVal genFileName(uchar **ppName, uchar *pDirName, size_t lenDirName, uchar *
 		lenBuf = 0;
 	} else {
 		if(lNumDigits > 0) {
-			snprintf(szFmtBuf, sizeof(szFmtBuf), ".%%0%dld", lNumDigits);
+			snprintf(szFmtBuf, sizeof(szFmtBuf), ".%%0%d" PRId64, lNumDigits);
 			lenBuf = snprintf((char*)szBuf, sizeof(szBuf), szFmtBuf, lNum);
 		} else
-			lenBuf = snprintf((char*)szBuf, sizeof(szBuf), ".%ld", lNum);
+			lenBuf = snprintf((char*)szBuf, sizeof(szBuf), ".%" PRId64, lNum);
 	}
 
 	lenName = lenDirName + 1 + lenFName + lenBuf + 1; /* last +1 for \0 char! */
@@ -378,7 +388,9 @@ rsRetVal genFileName(uchar **ppName, uchar *pDirName, size_t lenDirName, uchar *
 finalize_it:
 	RETiRet;
 }
+#if !defined(_AIX)
 #pragma GCC diagnostic pop
+#endif
 
 /* get the number of digits required to represent a given number. We use an
  * iterative approach as we do not like to draw in the floating point
@@ -434,9 +446,8 @@ timeoutComp(struct timespec *pt, long iTimeout)
 long long
 currentTimeMills(void)
 {
-#	if _POSIX_TIMERS > 0
 	struct timespec tm;
-#   else
+#	if _POSIX_TIMERS <= 0
 	struct timeval tv;
 #	endif
 
