@@ -16,7 +16,7 @@
  * it turns out to be problematic. Then, we need to quasi-refcount the number of accesses
  * to the object.
  *
- * Copyright 2008-2016 Rainer Gerhards and Adiscon GmbH.
+ * Copyright 2008-2017 Rainer Gerhards and Adiscon GmbH.
  *
  * This file is part of the rsyslog runtime library.
  *
@@ -49,6 +49,10 @@
 #include <sys/stat.h>	 /* required for HP UX */
 #include <errno.h>
 #include <pthread.h>
+#include <poll.h>
+#ifdef HAVE_SYS_PRCTL_H
+#  include <sys/prctl.h>
+#endif
 
 #include "rsyslog.h"
 #include "stringbuf.h"
@@ -59,9 +63,7 @@
 #include "module-template.h"
 #include "errmsg.h"
 #include "cryprov.h"
-#ifdef HAVE_SYS_PRCTL_H
-#  include <sys/prctl.h>
-#endif
+#include "datetime.h"
 
 /* some platforms do not have large file support :( */
 #ifndef O_LARGEFILE
@@ -78,7 +80,8 @@ DEFobjCurrIf(zlibw)
 
 /* forward definitions */
 static rsRetVal strmFlushInternal(strm_t *pThis, int bFlushZip);
-static rsRetVal strmWrite(strm_t *__restrict__ const pThis, const uchar *__restrict__ const pBuf, const size_t lenBuf);
+static rsRetVal strmWrite(strm_t *__restrict__ const pThis, const uchar *__restrict__ const pBuf,
+	const size_t lenBuf);
 static rsRetVal strmCloseFile(strm_t *pThis);
 static void *asyncWriterThread(void *pPtr);
 static rsRetVal doZipWrite(strm_t *pThis, uchar *pBuf, size_t lenBuf, int bFlush);
@@ -161,9 +164,11 @@ resolveFileSizeLimit(strm_t *pThis, uchar *pszCurrFName)
 finalize_it:
 	if(iRet != RS_RET_OK) {
 		if(iRet == RS_RET_SIZELIMITCMD_DIDNT_RESOLVE) {
-			DBGPRINTF("file size limit cmd for file '%s' did no resolve situation\n", pszCurrFName);
+			LogError(0, RS_RET_ERR, "file size limit cmd for file '%s' "
+				"did no resolve situation\n", pszCurrFName);
 		} else {
-			DBGPRINTF("file size limit cmd for file '%s' failed with code %d.\n", pszCurrFName, iRet);
+			LogError(0, RS_RET_ERR, "file size limit cmd for file '%s' "
+				"failed with code %d.\n", pszCurrFName, iRet);
 		}
 		pThis->bDisabled = 1;
 	}
@@ -240,17 +245,14 @@ doPhysOpen(strm_t *pThis)
 	}
 
 	pThis->fd = open((char*)pThis->pszCurrFName, iFlags | O_LARGEFILE, pThis->tOpenMode);
+	const int errno_save = errno; /* dbgprintf can mangle it! */
 	DBGPRINTF("file '%s' opened as #%d with mode %d\n", pThis->pszCurrFName,
 		  pThis->fd, (int) pThis->tOpenMode);
 	if(pThis->fd == -1) {
-		char errStr[1024];
-		int err = errno;
-		rs_strerror_r(err, errStr, sizeof(errStr));
-		DBGOPRINT((obj_t*) pThis, "open error %d, file '%s': %s\n", errno, pThis->pszCurrFName, errStr);
-		if(err == ENOENT)
-			ABORT_FINALIZE(RS_RET_FILE_NOT_FOUND);
-		else
-			ABORT_FINALIZE(RS_RET_FILE_OPEN_ERROR);
+		const rsRetVal errcode = (errno_save == ENOENT)
+			? RS_RET_FILE_NOT_FOUND : RS_RET_FILE_OPEN_ERROR;
+		LogError(errno_save, errcode, "file '%s': open error", pThis->pszCurrFName);
+		ABORT_FINALIZE(errcode);
 	}
 
 	if(pThis->tOperationsMode == STREAMMODE_READ) {
@@ -357,16 +359,9 @@ static rsRetVal strmOpenFile(strm_t *pThis)
 		pThis->iCurrOffs = offset;
 	} else if(pThis->tOperationsMode == STREAMMODE_WRITE) {
 		if(offset != 0) {
-			// TODO: check the exact condition under which this occurs. It
-			// looks OK, seems like the queue code uses a side-effect that makes
-			// this a valid sequence!
-			/*errmsg.LogError(0, 0, "queue '%s', file '%s' opened for non-append write, but "
+			LogError(0, 0, "queue '%s', file '%s' opened for non-append write, but "
 				"already contains %zd bytes\n",
 				obj.GetName((obj_t*) pThis), pThis->pszCurrFName, offset);
-			*/
-			DBGPRINTF("queue '%s', file '%s' opened for non-append write, but "
-				"already contains %lld bytes\n",
-				obj.GetName((obj_t*) pThis), pThis->pszCurrFName, (long long) offset);
 		}
 	}
 
@@ -392,7 +387,7 @@ finalize_it:
  * that require data to be persisted. May be called in non-async mode and is a null
  * operation than. Must be called with the mutex locked.
  */
-static inline void
+static void
 strmWaitAsyncWriterDone(strm_t *pThis)
 {
 	BEGINfunc
@@ -493,9 +488,10 @@ strmNextFile(strm_t *pThis)
 {
 	DEFiRet;
 
-	ASSERT(pThis != NULL);
-	ASSERT(pThis->iMaxFiles != 0);
-	ASSERT(pThis->fd != -1);
+	assert(pThis != NULL);
+	assert(pThis->sType == STREAMTYPE_FILE_CIRCULAR);
+	assert(pThis->iMaxFiles != 0);
+	assert(pThis->fd != -1);
 
 	CHKiRet(strmCloseFile(pThis));
 
@@ -656,9 +652,9 @@ strmDebugOutBuf(const strm_t *const pThis)
 	int strtIdx = pThis->iBufPtr - 50;
 	if(strtIdx < 0)
 		strtIdx = 0;
-	DBGOPRINT((obj_t*) pThis, "strmRead index %zd, max %zd, buf '%.*s'\n",
-		pThis->iBufPtr, pThis->iBufPtrMax, (int) pThis->iBufPtrMax - strtIdx,
-		pThis->pIOBuf+strtIdx);
+	DBGOPRINT((obj_t*) pThis, "strmRead ungetc %d, index %zd, max %zd, buf '%.*s', CURR: '%.*s'\n",
+		pThis->iUngetC, pThis->iBufPtr, pThis->iBufPtrMax, (int) pThis->iBufPtrMax - strtIdx,
+		pThis->pIOBuf+strtIdx, (int) (pThis->iBufPtrMax - pThis->iBufPtr), pThis->pIOBuf+pThis->iBufPtr);
 }
 
 /* logically "read" a character from a file. What actually happens is that
@@ -752,9 +748,10 @@ strmReadLine(strm_t *pThis, cstr_t **ppCStr, uint8_t mode, sbool bEscapeLF, uint
 	}
         if(mode == 0) {
 		while(c != '\n') {
-                	CHKiRet(cstrAppendChar(*ppCStr, c));
-                	readCharRet = strmReadChar(pThis, &c);
-                	if(readCharRet == RS_RET_EOF) {/* end of file reached without \n? */
+			CHKiRet(cstrAppendChar(*ppCStr, c));
+			readCharRet = strmReadChar(pThis, &c);
+			if((readCharRet == RS_RET_TIMED_OUT) ||
+			   (readCharRet == RS_RET_EOF) ) { /* end reached without \n? */
 				CHKiRet(rsCStrConstructFromCStr(&pThis->prevLineSegment, *ppCStr));
                 	}
                 	CHKiRet(readCharRet);
@@ -857,6 +854,20 @@ finalize_it:
         RETiRet;
 }
 
+/* check if the current multi line read is timed out
+ * @return 0 - no timeout, something else - timeout
+ */
+int
+strmReadMultiLine_isTimedOut(const strm_t *const __restrict__ pThis)
+{
+	/* note: order of evaluation is choosen so that the most inexpensive
+	 * processing flow is used.
+	 */
+	return(   (pThis->readTimeout)
+	       && (pThis->prevMsgSegment != NULL)
+	       && (getTime(NULL) > pThis->lastRead + pThis->readTimeout) );
+}
+
 /* read a multi-line message from a strm file.
  * The multi-line message is terminated based on the user-provided
  * startRegex (Posix ERE). For performance reasons, the regex
@@ -864,19 +875,18 @@ finalize_it:
  * added 2015-05-12 rgerhards
  */
 rsRetVal
-strmReadMultiLine(strm_t *pThis, cstr_t **ppCStr, regex_t *preg, sbool bEscapeLF)
+strmReadMultiLine(strm_t *pThis, cstr_t **ppCStr, regex_t *preg, const sbool bEscapeLF)
 {
         uchar c;
 	uchar finished = 0;
 	cstr_t *thisLine = NULL;
 	rsRetVal readCharRet;
+	const time_t tCurr = pThis->readTimeout ? getTime(NULL) : 0;
         DEFiRet;
 
-        ASSERT(pThis != NULL);
-        ASSERT(ppCStr != NULL);
-
 	do {
 		CHKiRet(strmReadChar(pThis, &c)); /* immediately exit on EOF */
+		pThis->lastRead = tCurr;
 		CHKiRet(cstrConstruct(&thisLine));
 		/* append previous message to current message if necessary */
 		if(pThis->prevLineSegment != NULL) {
@@ -909,15 +919,19 @@ strmReadMultiLine(strm_t *pThis, cstr_t **ppCStr, regex_t *preg, sbool bEscapeLF
 			CHKiRet(rsCStrConstructFromCStr(&pThis->prevMsgSegment, thisLine));
 			
 		} else {
-			if(pThis->prevMsgSegment != NULL) {
-				/* may be NULL in initial poll! */
+			if(pThis->prevMsgSegment == NULL) {
+				/* may be NULL in initial poll or after timeout! */
+				CHKiRet(rsCStrConstructFromCStr(&pThis->prevMsgSegment, thisLine));
+			} else {
 				if(bEscapeLF) {
 					rsCStrAppendStrWithLen(pThis->prevMsgSegment, (uchar*)"\\n", 2);
 				} else {
 					cstrAppendChar(pThis->prevMsgSegment, '\n');
 				}
-				CHKiRet(cstrAppendCStr(pThis->prevMsgSegment, thisLine));
-				/* we could do this faster, but for now keep it simple */
+				if(cstrLen(thisLine) > 0) {
+					CHKiRet(cstrAppendCStr(pThis->prevMsgSegment, thisLine));
+					/* we could do this faster, but for now keep it simple */
+				}
 
 			}
 		}
@@ -925,6 +939,17 @@ strmReadMultiLine(strm_t *pThis, cstr_t **ppCStr, regex_t *preg, sbool bEscapeLF
 	} while(finished == 0);
 
 finalize_it:
+	if(   pThis->readTimeout
+	   && (iRet != RS_RET_OK)
+	   && (pThis->prevMsgSegment != NULL)
+	   && (tCurr > pThis->lastRead + pThis->readTimeout)) {
+		CHKiRet(rsCStrConstructFromCStr(ppCStr, pThis->prevMsgSegment));
+		cstrDestruct(&pThis->prevMsgSegment);
+		pThis->lastRead = tCurr;
+		dbgprintf("stream: generated msg based on timeout: %s\n", cstrGetSzStrNoNULL(*ppCStr));
+			FINALIZE;
+		iRet = RS_RET_OK;
+	}
         RETiRet;
 }
 
@@ -1023,7 +1048,7 @@ finalize_it:
 /* stop the writer thread (we MUST be runnnig asynchronously when this method
  * is called!). Note that the mutex must be locked! -- rgerhards, 2009-07-06
  */
-static inline void
+static void
 stopWriter(strm_t *pThis)
 {
 	BEGINfunc
@@ -1088,7 +1113,7 @@ static rsRetVal strmCheckNextOutputFile(strm_t *pThis)
 {
 	DEFiRet;
 
-	if(pThis->fd == -1)
+	if(pThis->fd == -1 || pThis->sType != STREAMTYPE_FILE_CIRCULAR)
 		FINALIZE;
 
 	/* wait for output to be empty, so that our counts are correct */
@@ -1121,7 +1146,10 @@ tryTTYRecover(strm_t *pThis, int err)
 #ifndef __FreeBSD__
 	if(err == ERR_TTYHUP) {
 #else
-	if(err == ERR_TTYHUP || err == ENXIO) {
+	/* Try to reopen our file descriptor even on errno 6, FreeBSD bug 200429
+	 * Also try on errno 5, FreeBSD bug 211033
+	 */
+	if(err == ERR_TTYHUP || err == ENXIO || err == EIO) {
 #endif /* __FreeBSD__ */
 		close(pThis->fd);
 		CHKiRet(doPhysOpen(pThis));
@@ -1172,13 +1200,17 @@ doWriteCall(strm_t *pThis, uchar *pBuf, size_t *pLenBuf)
 #endif /* __FreeBSD__ */
 		iWritten = write(pThis->fd, pWriteBuf, lenBuf);
 		if(iWritten < 0) {
-			char errStr[1024];
-			int err = errno;
+			const int err = errno;
 			iWritten = 0; /* we have written NO bytes! */
-			rs_strerror_r(err, errStr, sizeof(errStr));
-			DBGPRINTF("log file (%d) write error %d: %s\n", pThis->fd, err, errStr);
+			if(err != EINTR) {
+				LogError(err, RS_RET_IO_ERROR, "file '%d' write error", pThis->fd);
+			}
 			if(err == EINTR) {
 				/*NO ERROR, just continue */;
+			} else if( !pThis->bIsTTY && ( err == ENOTCONN  || err == EIO )) {
+				/* Failure for network file system, thus file needs to be closed and reopened. */
+				close(pThis->fd);
+				CHKiRet(doPhysOpen(pThis));
 			} else {
 				if(pThis->bIsTTY) {
 					CHKiRet(tryTTYRecover(pThis, err));
@@ -1207,7 +1239,7 @@ finalize_it:
 
 /* write memory buffer to a stream object.
  */
-static inline rsRetVal
+static rsRetVal
 doWriteInternal(strm_t *pThis, uchar *pBuf, const size_t lenBuf, const int bFlush)
 {
 	DEFiRet;
@@ -1235,7 +1267,7 @@ finalize_it:
  * the very some producer comes back in sequence to submit the then-filled buffers.
  * This also enables us to timout on partially written buffers. -- rgerhards, 2009-07-06
  */
-static inline rsRetVal
+static rsRetVal
 doAsyncWriteInternal(strm_t *pThis, size_t lenBuf, const int bFlushZip)
 {
 	DEFiRet;
@@ -1498,7 +1530,7 @@ doZipWrite(strm_t *pThis, uchar *pBuf, size_t lenBuf, const int bFlush)
 {
 	int zRet;	/* zlib return state */
 	DEFiRet;
-	unsigned outavail;
+	unsigned outavail = 0;
 	assert(pThis != NULL);
 	assert(pBuf != NULL);
 
@@ -1510,7 +1542,7 @@ doZipWrite(strm_t *pThis, uchar *pBuf, size_t lenBuf, const int bFlush)
 		/* see note in file header for the params we use with deflateInit2() */
 		zRet = zlibw.DeflateInit2(&pThis->zstrm, pThis->iZipLevel, Z_DEFLATED, 31, 9, Z_DEFAULT_STRATEGY);
 		if(zRet != Z_OK) {
-			DBGPRINTF("error %d returned from zlib/deflateInit2()\n", zRet);
+			LogError(0, RS_RET_ZLIB_ERR, "error %d returned from zlib/deflateInit2()", zRet);
 			ABORT_FINALIZE(RS_RET_ZLIB_ERR);
 		}
 		pThis->bzInitDone = RSTRUE;
@@ -1526,9 +1558,13 @@ doZipWrite(strm_t *pThis, uchar *pBuf, size_t lenBuf, const int bFlush)
 		pThis->zstrm.avail_out = pThis->sIOBufSize;
 		pThis->zstrm.next_out = pThis->pZipBuf;
 		zRet = zlibw.Deflate(&pThis->zstrm, bFlush ? Z_SYNC_FLUSH : Z_NO_FLUSH);    /* no bad return value */
-		outavail =pThis->sIOBufSize - pThis->zstrm.avail_out;
 		DBGPRINTF("after deflate, ret %d, avail_out %d, to write %d\n",
 			zRet, pThis->zstrm.avail_out, outavail);
+		if(zRet != Z_OK) {
+			LogError(0, RS_RET_ZLIB_ERR, "error %d returned from zlib/Deflate()", zRet);
+			ABORT_FINALIZE(RS_RET_ZLIB_ERR);
+		}
+		outavail = pThis->sIOBufSize - pThis->zstrm.avail_out;
 		if(outavail != 0) {
 			CHKiRet(strmPhysWrite(pThis, (uchar*)pThis->pZipBuf, outavail));
 		}
@@ -1574,7 +1610,7 @@ doZipFinish(strm_t *pThis)
 finalize_it:
 	zRet = zlibw.DeflateEnd(&pThis->zstrm);
 	if(zRet != Z_OK) {
-		DBGPRINTF("error %d returned from zlib/deflateEnd()\n", zRet);
+		LogError(0, RS_RET_ZLIB_ERR, "error %d returned from zlib/deflateEnd()", zRet);
 	}
 
 	pThis->bzInitDone = 0;
@@ -1668,7 +1704,7 @@ finalize_it:
  * rgerhards, 2012-11-07
  */
 rsRetVal
-strmMultiFileSeek(strm_t *pThis, int FNum, off64_t offs, off64_t *bytesDel)
+strmMultiFileSeek(strm_t *pThis, unsigned int FNum, off64_t offs, off64_t *bytesDel)
 {
 	struct stat statBuf;
 	DEFiRet;
@@ -1690,7 +1726,7 @@ strmMultiFileSeek(strm_t *pThis, int FNum, off64_t offs, off64_t *bytesDel)
 				    pThis->iFileNumDigits));
 		stat((char*)pThis->pszCurrFName, &statBuf);
 		*bytesDel = statBuf.st_size;
-		DBGPRINTF("strmMultiFileSeek: detected new filenum, was %d, new %d, "
+		DBGPRINTF("strmMultiFileSeek: detected new filenum, was %u, new %u, "
 			  "deleting '%s' (%lld bytes)\n", pThis->iCurrFNum, FNum,
 			  pThis->pszCurrFName, (long long) *bytesDel);
 		unlink((char*)pThis->pszCurrFName);
@@ -1814,7 +1850,9 @@ strmWrite(strm_t *__restrict__ const pThis, const uchar *__restrict__ const pBuf
 	ASSERT(pThis != NULL);
 	ASSERT(pBuf != NULL);
 
-	/* DEV DEBUG ONLY DBGPRINTF("strmWrite(%p[%s], '%65.65s', %ld);, disabled %d, sizelim %ld, size %lld\n", pThis, pThis->pszCurrFName, pBuf,(long) lenBuf, pThis->bDisabled, (long) pThis->iSizeLimit, (long long) pThis->iCurrOffs); */
+/* DEV DEBUG ONLY DBGPRINTF("strmWrite(%p[%s], '%65.65s', %ld);,
+disabled %d, sizelim %ld, size %lld\n", pThis, pThis->pszCurrFName, pBuf,(long) lenBuf,
+pThis->bDisabled, (long) pThis->iSizeLimit, (long long) pThis->iCurrOffs); */
 	if(pThis->bDisabled)
 		ABORT_FINALIZE(RS_RET_STREAM_DISABLED);
 
@@ -1876,6 +1914,13 @@ DEFpropSetMeth(strm, pszSizeLimitCmd, uchar*)
 DEFpropSetMeth(strm, cryprov, cryprov_if_t*)
 DEFpropSetMeth(strm, cryprovData, void*)
 
+/* sets timeout in seconds */
+void
+strmSetReadTimeout(strm_t *const __restrict__ pThis, const int val)
+{
+	pThis->readTimeout = val;
+}
+
 static rsRetVal strmSetbDeleteOnClose(strm_t *pThis, int val)
 {
 	pThis->bDeleteOnClose = val;
@@ -2013,7 +2058,7 @@ static rsRetVal strmSerialize(strm_t *pThis, strm_t *pStrm)
 	strmFlushInternal(pThis, 0);
 	CHKiRet(obj.BeginSerialize(pStrm, (obj_t*) pThis));
 
-	objSerializeSCALAR(pStrm, iCurrFNum, INT);
+	objSerializeSCALAR(pStrm, iCurrFNum, INT); /* implicit cast is OK for persistance */
 	objSerializePTR(pStrm, pszFName, PSZ);
 	objSerializeSCALAR(pStrm, iMaxFiles, INT);
 	objSerializeSCALAR(pStrm, bDeleteOnClose, INT);
@@ -2134,7 +2179,7 @@ static rsRetVal strmSetProperty(strm_t *pThis, var_t *pProp)
  	if(isProp("sType")) {
 		CHKiRet(strmSetsType(pThis, (strmType_t) pProp->val.num));
  	} else if(isProp("iCurrFNum")) {
-		pThis->iCurrFNum = pProp->val.num;
+		pThis->iCurrFNum = (unsigned) pProp->val.num;
  	} else if(isProp("pszFName")) {
 		CHKiRet(strmSetFName(pThis, rsCStrGetSzStrNoNULL(pProp->val.pStr), rsCStrLen(pProp->val.pStr)));
  	} else if(isProp("tOperationsMode")) {
