@@ -75,7 +75,9 @@
 #include "ratelimit.h"
 #include "unicode-helper.h"
 
+#if !defined(_AIX)
 #pragma GCC diagnostic ignored "-Wswitch-enum"
+#endif
 
 MODULE_TYPE_LIB
 MODULE_TYPE_NOKEEP
@@ -123,7 +125,7 @@ static int wrkrRunning;
 /* add new listener port to listener port list
  * rgerhards, 2009-05-21
  */
-static inline rsRetVal
+static rsRetVal
 addNewLstnPort(tcpsrv_t *pThis, uchar *pszPort, int bSuppOctetFram, uchar *pszAddr)
 {
 	tcpLstnPortList_t *pEntry;
@@ -361,7 +363,7 @@ finalize_it:
 /* Initialize TCP listener socket for a single port
  * rgerhards, 2009-05-21
  */
-static inline rsRetVal
+static rsRetVal
 initTCPListener(tcpsrv_t *pThis, tcpLstnPortList_t *pPortEntry)
 {
 	DEFiRet;
@@ -382,7 +384,8 @@ initTCPListener(tcpsrv_t *pThis, tcpLstnPortList_t *pPortEntry)
 		TCPLstnPort = pPortEntry->pszPort;
 
 	// pPortEntry->pszAddr = NULL ==> bind to all interfaces
-        CHKiRet(netstrm.LstnInit(pThis->pNS, (void*)pPortEntry, addTcpLstn, TCPLstnPort, pPortEntry->pszAddr, pThis->iSessMax));
+        CHKiRet(netstrm.LstnInit(pThis->pNS, (void*)pPortEntry, addTcpLstn, TCPLstnPort,
+	pPortEntry->pszAddr, pThis->iSessMax));
 
 finalize_it:
 	RETiRet;
@@ -404,7 +407,8 @@ create_tcp_socket(tcpsrv_t *pThis)
 	while(pEntry != NULL) {
 		localRet = initTCPListener(pThis, pEntry);
 		if(localRet != RS_RET_OK) {
-			errmsg.LogError(0, localRet, "Could not create tcp listener, ignoring port %s bind-address %s.", pEntry->pszPort, pEntry->pszAddr);
+			errmsg.LogError(0, localRet, "Could not create tcp listener, ignoring port "
+			"%s bind-address %s.", pEntry->pszPort, pEntry->pszAddr);
 		}
 		pEntry = pEntry->pNext;
 	}
@@ -540,7 +544,7 @@ RunCancelCleanup(void *arg)
 /* helper to close a session. Takes status of poll vs. select into consideration.
  * rgerhards, 2009-11-25
  */
-static inline rsRetVal
+static rsRetVal
 closeSess(tcpsrv_t *pThis, tcps_sess_t **ppSess, nspoll_t *pPoll) {
 	DEFiRet;
 	if(pPoll != NULL) {
@@ -613,7 +617,7 @@ finalize_it:
 
 /* process a single workset item
  */
-static inline rsRetVal
+static rsRetVal
 processWorksetItem(tcpsrv_t *pThis, nspoll_t *pPoll, int idx, void *pUsr)
 {
 	tcps_sess_t *pNewSess = NULL;
@@ -696,7 +700,7 @@ processWorkset(tcpsrv_t *pThis, nspoll_t *pPoll, int numEntries, nsd_epworkset_t
 			ABORT_FINALIZE(RS_RET_FORCE_TERM);
 		if(numEntries == 1) {
 			/* process self, save context switch */
-			processWorksetItem(pThis, pPoll, workset[numEntries-1].id, workset[numEntries-1].pUsr);
+			iRet = processWorksetItem(pThis, pPoll, workset[numEntries-1].id, workset[numEntries-1].pUsr);
 		} else {
 			pthread_mutex_lock(&wrkrMut);
 			/* check if there is a free worker */
@@ -719,7 +723,7 @@ processWorkset(tcpsrv_t *pThis, nspoll_t *pPoll, int numEntries, nsd_epworkset_t
 			} else {
 				pthread_mutex_unlock(&wrkrMut);
 				/* no free worker, so we process this one ourselfs */
-				processWorksetItem(pThis, pPoll, workset[numEntries-1].id,
+				iRet = processWorksetItem(pThis, pPoll, workset[numEntries-1].id,
 						   workset[numEntries-1].pUsr);
 			}
 		}
@@ -747,7 +751,9 @@ finalize_it:
  * This variant here is only used if we need to work with a netstream driver
  * that does not support epoll().
  */
+#if !defined(_AIX)
 #pragma GCC diagnostic ignored "-Wempty-body"
+#endif
 static rsRetVal
 RunSelect(tcpsrv_t *pThis, nsd_epworkset_t workset[], size_t sizeWorkset)
 {
@@ -851,7 +857,9 @@ finalize_it: /* this is a very special case - this time only we do not exit the
 
 	RETiRet;
 }
+#if !defined(_AIX)
 #pragma GCC diagnostic warning "-Wempty-body"
+#endif
 
 
 /* This function is called to gather input. It tries doing that via the epoll()
@@ -864,6 +872,7 @@ Run(tcpsrv_t *pThis)
 {
 	DEFiRet;
 	int i;
+	int bFailed = FALSE; /* If set to TRUE, accept failed already */
 	nsd_epworkset_t workset[128]; /* 128 is currently fixed num of concurrent requests */
 	int numEntries;
 	nspoll_t *pPoll = NULL;
@@ -922,7 +931,23 @@ Run(tcpsrv_t *pThis)
 		if(localRet != RS_RET_OK)
 			continue;
 
-		processWorkset(pThis, pPoll, numEntries, workset);
+		localRet = processWorkset(pThis, pPoll, numEntries, workset);
+		if(localRet != RS_RET_OK) {
+			if (bFailed == FALSE) {
+				errmsg.LogError(0, localRet, "tcpsrv listener (inputname: '%s') failed "
+				"to processed incoming connection with error %d",
+				(pThis->pszInputName == NULL) ? (uchar*)"*UNSET*" : pThis->pszInputName, localRet);
+				bFailed = TRUE; 
+			} else {
+				DBGPRINTF("tcpsrv listener (inputname: '%s') still failing to process incoming connection with error %d\n",
+					(pThis->pszInputName == NULL) ? (uchar*)"*UNSET*" : pThis->pszInputName, localRet);
+			}
+			/* Sleep 20ms */
+			srSleep(0,20000); 
+		} else {
+			/* Reset bFailed State */
+			bFailed = FALSE; 
+		}
 	}
 
 	/* remove the tcp listen sockets from the epoll set */
@@ -1029,7 +1054,11 @@ SetCBRcvData(tcpsrv_t *pThis, rsRetVal (*pRcvData)(tcps_sess_t*, char*, size_t,
 }
 
 static rsRetVal
+#ifdef _AIX
+SetCBOnListenDeinit(tcpsrv_t *pThis, rsRetVal (*pCB)(void*))
+#else
 SetCBOnListenDeinit(tcpsrv_t *pThis, int (*pCB)(void*))
+#endif
 {
 	DEFiRet;
 	pThis->pOnListenDeinit = pCB;
