@@ -48,8 +48,9 @@
 #include "parserif.h"
 #include "unicode-helper.h"
 
+#if !defined(_AIX)
 #pragma GCC diagnostic ignored "-Wswitch-enum"
-
+#endif
 /* static data */
 DEFobjCurrIf(obj)
 DEFobjCurrIf(errmsg)
@@ -151,7 +152,7 @@ finalize_it:
  */
 rsRetVal
 tplToString(struct template *__restrict__ const pTpl,
-	    msg_t *__restrict__ const pMsg,
+	    smsg_t *__restrict__ const pMsg,
 	    actWrkrIParams_t *__restrict const iparam,
 	    struct syslogTime *const ttNow)
 {
@@ -259,7 +260,7 @@ finalize_it:
  * rgerhards, 2009-04-03
  */
 rsRetVal
-tplToArray(struct template *pTpl, msg_t *pMsg, uchar*** ppArr, struct syslogTime *ttNow)
+tplToArray(struct template *pTpl, smsg_t *pMsg, uchar*** ppArr, struct syslogTime *ttNow)
 {
 	DEFiRet;
 	struct templateEntry *pTpe;
@@ -330,7 +331,7 @@ finalize_it:
  * rgerhards, 2012-08-29
  */
 rsRetVal
-tplToJSON(struct template *pTpl, msg_t *pMsg, struct json_object **pjson, struct syslogTime *ttNow)
+tplToJSON(struct template *pTpl, smsg_t *pMsg, struct json_object **pjson, struct syslogTime *ttNow)
 {
 	struct templateEntry *pTpe;
 	rs_size_t propLen;
@@ -411,12 +412,17 @@ finalize_it:
 static void doEmergencyEscape(register uchar *p, int mode)
 {
 	while(*p) {
-		if((mode == SQL_ESCAPE||mode == STDSQL_ESCAPE) && *p == '\'')
+		if((mode == SQL_ESCAPE||mode == STDSQL_ESCAPE) && *p == '\'') {
 			*p = '"';
-		else if((mode == JSON_ESCAPE) && *p == '"')
-			*p = '\'';
-		else if((mode == SQL_ESCAPE) && *p == '\\')
+		} else if(mode == JSON_ESCAPE) {
+			if(*p == '"') {
+				*p = '\'';
+			} else if(*p == '\\' ) {
+				*p = '/';
+			}
+		} else if((mode == SQL_ESCAPE) && *p == '\\') {
 			*p = '/';
+		}
 		++p;
 	}
 }
@@ -467,7 +473,7 @@ doEscape(uchar **pp, rs_size_t *pLen, unsigned short *pbMustBeFreed, int mode)
 		for(p = *pp ; *p && *p != '\'' && *p != '\\' ; ++p)
 			;
 	else if(mode == JSON_ESCAPE)
-		for(p = *pp ; *p && *p != '"' ; ++p)
+		for(p = *pp ; *p &&  (*p == '"' || *p == '\\' ) ; ++p)
 			;
 	/* when we get out of the loop, we are either at the
 	 * string terminator or the first character to escape */
@@ -485,7 +491,7 @@ doEscape(uchar **pp, rs_size_t *pLen, unsigned short *pbMustBeFreed, int mode)
 		} else if((mode == SQL_ESCAPE) && *p == '\\') {
 			CHKiRet(cstrAppendChar(pStrB, '\\'));
 			iLen++;	/* reflect the extra character */
-		} else if((mode == JSON_ESCAPE) && *p == '"') {
+		} else if((mode == JSON_ESCAPE) &&  (*p == '"' || *p == '\\' )) {
 			CHKiRet(cstrAppendChar(pStrB, '\\'));
 			iLen++;	/* reflect the extra character */
 		}
@@ -997,7 +1003,8 @@ do_Parameter(uchar **pp, struct template *pTpl)
 					pTpe->data.field.has_fields = 1;
 					if(!isdigit((int)*p)) {
 						/* complain and use default */
-						errmsg.LogError(0, NO_ERRCODE, "error: invalid character in frompos after \"F,\", property: '%%%s' - using 9 (HT) as field delimiter",
+						errmsg.LogError(0, NO_ERRCODE, "error: invalid character in "
+"frompos after \"F,\", property: '%%%s' - using 9 (HT) as field delimiter",
 						    (char*) *pp);
 						pTpe->data.field.field_delim = 9;
 					} else {
@@ -1005,7 +1012,8 @@ do_Parameter(uchar **pp, struct template *pTpl)
 						while(isdigit((int)*p))
 							iNum = iNum * 10 + *p++ - '0';
 						if(iNum < 0 || iNum > 255) {
-							errmsg.LogError(0, NO_ERRCODE, "error: non-USASCII delimiter character value %d in template - using 9 (HT) as substitute", iNum);
+							errmsg.LogError(0, NO_ERRCODE, "error: non-USASCII delimiter "
+"character value %d in template - using 9 (HT) as substitute", iNum);
 							pTpe->data.field.field_delim = 9;
 						} else {
 							pTpe->data.field.field_delim = iNum;
