@@ -71,7 +71,8 @@ case $1 in
 		rm -f work rsyslog.out.log rsyslog2.out.log rsyslog.out.log.save # common work files
 		rm -rf test-spool test-logdir stat-file1
 		rm -f rsyslog.out.*.log work-presort rsyslog.pipe
-		rm -f rsyslog.input rsyslog.empty
+		rm -f -r rsyslog.input.*
+		rm -f rsyslog.input rsyslog.empty rsyslog.input.* imfile-state*
 		rm -f testconf.conf HOSTNAME
 		rm -f rsyslog.errorfile tmp.qi
 		rm -f core.* vgcore.*
@@ -116,7 +117,8 @@ case $1 in
 		rm -f work rsyslog.out.log rsyslog2.out.log rsyslog.out.log.save # common work files
 		rm -rf test-spool test-logdir stat-file1
 		rm -f rsyslog.out.*.log rsyslog.random.data work-presort rsyslog.pipe
-		rm -f rsyslog.input rsyslog.conf.tlscert stat-file1 rsyslog.empty
+		rm -f -r rsyslog.input.*
+		rm -f rsyslog.input rsyslog.conf.tlscert stat-file1 rsyslog.empty rsyslog.input.* imfile-state*
 		rm -f testconf.conf
 		rm -f rsyslog.errorfile tmp.qi
 		rm -f HOSTNAME imfile-state:.-rsyslog.input
@@ -210,7 +212,7 @@ case $1 in
 		i=0
 		while test ! -f rsyslogd$2.started; do
 			./msleep 100 # wait 100 milliseconds
-			ps -p `cat rsyslog$2.pid`
+			ps -p `cat rsyslog$2.pid` &> /dev/null
 			if [ $? -ne 0 ]
 			then
 			   echo "ABORT! rsyslog pid no longer active during startup!"
@@ -318,7 +320,7 @@ case $1 in
 		;;
    'shutdown-immediate') # shut rsyslogd down without emptying the queue. $2 is the instance.
 		cp rsyslog$2.pid rsyslog$2.pid.save
-		kill `cat rsyslog.pid`
+		kill `cat rsyslog$2.pid`
 		# note: we do not wait for the actual termination!
 		;;
    'kill-immediate') # kill rsyslog unconditionally
@@ -414,9 +416,11 @@ case $1 in
 		fi
 		;;
    'content-check-with-count') 
-		count=$(cat rsyslog.out.log | grep -qF "$2" | wc -l)
+		count=$(cat rsyslog.out.log | grep -F "$2" | wc -l)
 		if [ "x$count" == "x$3" ]; then
-		    echo content-check failed, expected $2 to occure $3 times, but found it $count times
+		    echo content-check-with-count success, \"$2\" occured $3 times
+		else
+		    echo content-check-with-count failed, expected \"$2\" to occure $3 times, but found it $count times
 		    . $srcdir/diag.sh error-exit 1
 		fi
 		;;
@@ -463,6 +467,8 @@ case $1 in
 		cat $3 | grep -qF "$2"
 		if [ "$?" -ne "0" ]; then
 		    echo content-check failed to find "'$2'" inside "'$3'"
+		    echo "file contents:"
+		    cat $3
 		    . $srcdir/diag.sh error-exit 1
 		fi
 		;;
@@ -470,6 +476,8 @@ case $1 in
 		sum=$(cat $4 | grep $3 | sed -e $2 | awk '{s+=$1} END {print s}')
 		if [ "x${sum}" != "x$5" ]; then
 		    echo sum of first column with edit-expr "'$2'" run over lines from file "'$4'" matched by "'$3'" equals "'$sum'" which is not equal to expected value of "'$5'"
+		    echo "file contents:"
+		    cat $4
 		    . $srcdir/diag.sh error-exit 1
 		fi
 		;;
@@ -477,6 +485,8 @@ case $1 in
 		sum=$(cat $4 | grep $3 | sed -e $2 | awk '{s+=$1} END {print s}')
 		if [ ! $sum -gt $5 ]; then
 		    echo sum of first column with edit-expr "'$2'" run over lines from file "'$4'" matched by "'$3'" equals "'$sum'" which is smaller than expected lower-limit of "'$5'"
+		    echo "file contents:"
+		    cat $4
 		    . $srcdir/diag.sh error-exit 1
 		fi
 		;;
@@ -484,6 +494,8 @@ case $1 in
 		cat rsyslog.out.log | grep -q "$2"
 		if [ "$?" -ne "0" ]; then
 		    echo content-check failed, not every line matched pattern "'$2'"
+		    echo "file contents:"
+		    cat $4
 		    . $srcdir/diag.sh error-exit 1
 		fi
 		;;
