@@ -37,6 +37,7 @@
 #include <ctype.h>
 #include <unistd.h>
 #include <stdint.h>
+#include <fcntl.h>
 #include <zlib.h>
 #include <pthread.h>
 #include "syslogd.h"
@@ -82,9 +83,12 @@ typedef struct _instanceData {
 	permittedPeers_t *pPermPeers;
 	int iStrmDrvrMode;
 	char	*target;
+	char	*device;
 	int compressionLevel;	/* 0 - no compression, else level for zlib */
 	char *port;
 	int protocol;
+	char *networkNamespace;
+	int originalNamespace;
 	int iRebindInterval;	/* rebind interval */
 	sbool bKeepAlive;
 	int iKeepAliveIntvl;
@@ -155,8 +159,10 @@ static struct cnfparamblk modpblk =
 /* action (instance) parameters */
 static struct cnfparamdescr actpdescr[] = {
 	{ "target", eCmdHdlrGetWord, 0 },
+	{ "device", eCmdHdlrGetWord, 0 },
 	{ "port", eCmdHdlrGetWord, 0 },
 	{ "protocol", eCmdHdlrGetWord, 0 },
+	{ "networknamespace", eCmdHdlrGetWord, 0 },
 	{ "tcp_framing", eCmdHdlrGetWord, 0 },
 	{ "ziplevel", eCmdHdlrInt, 0 },
 	{ "compression.mode", eCmdHdlrGetWord, 0 },
@@ -381,7 +387,9 @@ CODESTARTfreeInstance
 	free(pData->pszStrmDrvr);
 	free(pData->pszStrmDrvrAuthMode);
 	free(pData->port);
+	free(pData->networkNamespace);
 	free(pData->target);
+	free(pData->device);
 	net.DestructPermittedPeers(&pData->pPermPeers);
 ENDfreeInstance
 
@@ -415,6 +423,7 @@ static rsRetVal UDPSend(wrkrInstanceData_t *__restrict__ const pWrkrData,
 	int i;
 	unsigned lsent = 0;
 	sbool bSendSuccess;
+	sbool reInit = RSFALSE;
 	int lasterrno = ENOENT;
 	char errStr[1024];
 
@@ -444,6 +453,7 @@ static rsRetVal UDPSend(wrkrInstanceData_t *__restrict__ const pWrkrData,
 					bSendSuccess = RSTRUE;
 					break;
 				} else {
+					reInit = RSTRUE;
 					lasterrno = errno;
 					DBGPRINTF("sendto() error: %d = %s.\n",
 						lasterrno,
@@ -453,6 +463,12 @@ static rsRetVal UDPSend(wrkrInstanceData_t *__restrict__ const pWrkrData,
 			if (lsent == len && !pWrkrData->pData->bSendToAll)
 			       break;
 		}
+
+		/* one or more send failures; close sockets and re-init */
+		if (reInit == RSTRUE) {
+			CHKiRet(closeUDPSockets(pWrkrData));
+		}
+
 		/* finished looping */
 		if(bSendSuccess == RSTRUE) {
 			if(pWrkrData->pData->iUDPSendDelay > 0) {
@@ -557,7 +573,8 @@ TCPSendBufCompressed(wrkrInstanceData_t *pWrkrData, uchar *buf, unsigned len, sb
 		op = Z_NO_FLUSH;
 	/* run deflate() on buffer until everything has been compressed */
 	do {
-		DBGPRINTF("omfwd: in deflate() loop, avail_in %d, total_in %ld, isFlush %d\n", pWrkrData->zstrm.avail_in, pWrkrData->zstrm.total_in, bIsFlush);
+		DBGPRINTF("omfwd: in deflate() loop, avail_in %d, total_in %ld, isFlush %d\n",
+			pWrkrData->zstrm.avail_in, pWrkrData->zstrm.total_in, bIsFlush);
 		pWrkrData->zstrm.avail_out = sizeof(zipBuf);
 		pWrkrData->zstrm.next_out = zipBuf;
 		zRet = deflate(&pWrkrData->zstrm, op);    /* no bad return value */
@@ -707,7 +724,7 @@ static rsRetVal TCPSendInit(void *pvData)
 		}
 		/* params set, now connect */
 		CHKiRet(netstrm.Connect(pWrkrData->pNetstrm, glbl.GetDefPFFamily(),
-			(uchar*)pData->port, (uchar*)pData->target));
+			(uchar*)pData->port, (uchar*)pData->target, pData->device));
 
 		/* set keep-alive if enabled */
 		if(pData->bKeepAlive) {
@@ -728,6 +745,87 @@ finalize_it:
 }
 
 
+/* change to network namespace pData->networkNamespace and keep the file
+ * descriptor to the original namespace.
+ */
+static rsRetVal changeToNs(instanceData *pData)
+{
+	DEFiRet;
+#ifdef HAVE_SETNS
+	int iErr;
+	int destinationNs = -1;
+	char *nsPath = NULL;
+
+	if(pData->networkNamespace) {
+		/* keep file descriptor of original network namespace */
+		pData->originalNamespace = open("/proc/self/ns/net", O_RDONLY);
+		if (pData->originalNamespace < 0) {
+			errmsg.LogError(0, RS_RET_IO_ERROR, "omfwd: could not read /proc/self/ns/net\n");
+			ABORT_FINALIZE(RS_RET_IO_ERROR);
+		}
+
+		/* build network namespace path */
+		if (asprintf(&nsPath, "/var/run/netns/%s", pData->networkNamespace) == -1) {
+			errmsg.LogError(0, RS_RET_OUT_OF_MEMORY, "omfwd: asprintf failed\n");
+			ABORT_FINALIZE(RS_RET_OUT_OF_MEMORY);
+		}
+
+		/* keep file descriptor of destination network namespace */
+		destinationNs = open(nsPath, 0);
+		if (destinationNs < 0) {
+			errmsg.LogError(0, RS_RET_IO_ERROR, "omfwd: could not change to namespace '%s'\n",
+					pData->networkNamespace);
+			ABORT_FINALIZE(RS_RET_IO_ERROR);
+		}
+
+		/* actually change in the destination network namespace */
+		if((iErr = (setns(destinationNs, CLONE_NEWNET))) != 0) {
+			dbgprintf("could not change to namespace '%s': %d%s\n",
+				  pData->networkNamespace, iErr, gai_strerror(iErr));
+			ABORT_FINALIZE(RS_RET_IO_ERROR);
+		}
+		close(destinationNs);
+		free(nsPath);
+		dbgprintf("omfwd: changed to network namespace '%s'\n", pData->networkNamespace);
+	}
+
+finalize_it:
+#else /* #ifdef HAVE_SETNS */
+		dbgprintf("omfwd: OS does not support network namespaces\n");
+#endif /* #ifdef HAVE_SETNS */
+	RETiRet;
+}
+
+
+/* return to the original network namespace. This should be called after
+ * changeToNs().
+ */
+static rsRetVal returnToOriginalNs(instanceData *pData)
+{
+	DEFiRet;
+#ifdef HAVE_SETNS
+	int iErr;
+
+	/* only in case a network namespace is given and a file descriptor to
+	 * the original namespace exists */
+	if(pData->networkNamespace && pData->originalNamespace >= 0) {
+		/* actually change to the original network namespace */
+		if((iErr = (setns(pData->originalNamespace, CLONE_NEWNET))) != 0) {
+			dbgprintf("could not return to original namespace: %d%s\n",
+				  iErr, gai_strerror(iErr));
+			ABORT_FINALIZE(RS_RET_IO_ERROR);
+		}
+
+		close(pData->originalNamespace);
+		dbgprintf("omfwd: returned to original network namespace\n");
+	}
+
+finalize_it:
+#endif /* #ifdef HAVE_SETNS */
+	RETiRet;
+}
+
+
 /* try to resume connection if it is not ready
  * rgerhards, 2007-08-02
  */
@@ -757,17 +855,24 @@ static rsRetVal doTryResume(wrkrInstanceData_t *pWrkrData)
 		}
 		dbgprintf("%s found, resuming.\n", pData->target);
 		pWrkrData->f_addr = res;
-		pWrkrData->bIsConnected = 1;
 		if(pWrkrData->pSockArray == NULL) {
-			pWrkrData->pSockArray = net.create_udp_socket((uchar*)pData->target, NULL, 0, 0, 0);
+			CHKiRet(changeToNs(pData));
+			pWrkrData->pSockArray = net.create_udp_socket((uchar*)pData->target, NULL, 0, 0, 0, pData->device);
+			CHKiRet(returnToOriginalNs(pData));
+		}
+		if(pWrkrData->pSockArray != NULL) {
+			pWrkrData->bIsConnected = 1;
 		}
 	} else {
+		CHKiRet(changeToNs(pData));
 		CHKiRet(TCPSendInit((void*)pWrkrData));
+		CHKiRet(returnToOriginalNs(pData));
 	}
 
 finalize_it:
 	DBGPRINTF("omfwd: doTryResume %s iRet %d\n", pWrkrData->pData->target, iRet);
 	if(iRet != RS_RET_OK) {
+		returnToOriginalNs(pData);
 		if(pWrkrData->f_addr != NULL) {
 			freeaddrinfo(pWrkrData->f_addr);
 			pWrkrData->f_addr = NULL;
@@ -936,6 +1041,8 @@ setInstParamDefaults(instanceData *pData)
 {
 	pData->tplName = NULL;
 	pData->protocol = FORW_UDP;
+	pData->networkNamespace = NULL;
+	pData->originalNamespace = -1;
 	pData->tcp_framing = TCP_FRAMING_OCTET_STUFFING;
 	pData->pszStrmDrvr = NULL;
 	pData->pszStrmDrvrAuthMode = NULL;
@@ -985,6 +1092,8 @@ CODESTARTnewActInst
 			continue;
 		if(!strcmp(actpblk.descr[i].name, "target")) {
 			pData->target = es_str2cstr(pvals[i].val.d.estr, NULL);
+		} else if(!strcmp(actpblk.descr[i].name, "device")) {
+			pData->device = es_str2cstr(pvals[i].val.d.estr, NULL);
 		} else if(!strcmp(actpblk.descr[i].name, "port")) {
 			pData->port = es_str2cstr(pvals[i].val.d.estr, NULL);
 		} else if(!strcmp(actpblk.descr[i].name, "protocol")) {
@@ -1006,6 +1115,8 @@ CODESTARTnewActInst
 				free(str);
 				ABORT_FINALIZE(RS_RET_INVLD_PROTOCOL);
 			}
+		} else if(!strcmp(actpblk.descr[i].name, "networknamespace")) {
+			pData->networkNamespace = es_str2cstr(pvals[i].val.d.estr, NULL);
 		} else if(!strcmp(actpblk.descr[i].name, "tcp_framing")) {
 			if(!es_strcasebufcmp(pvals[i].val.d.estr, (uchar*)"traditional", 11)) {
 				pData->tcp_framing = TCP_FRAMING_OCTET_STUFFING;
@@ -1239,6 +1350,7 @@ CODE_STD_STRING_REQUESTparseSelectorAct(1)
 
 	pData->tcp_framing = tcp_framing;
 	pData->port = NULL;
+	pData->networkNamespace = NULL;
 	if(*p == ':') { /* process port */
 		uchar * tmp;
 
@@ -1366,19 +1478,32 @@ CODEmodInit_QueryRegCFSLineHdlr
 	CHKiRet(objUse(errmsg, CORE_COMPONENT));
 	CHKiRet(objUse(net,LM_NET_FILENAME));
 
-	CHKiRet(regCfSysLineHdlr((uchar *)"actionforwarddefaulttemplate", 0, eCmdHdlrGetWord, setLegacyDfltTpl, NULL, NULL));
-	CHKiRet(regCfSysLineHdlr((uchar *)"actionsendtcprebindinterval", 0, eCmdHdlrInt, NULL, &cs.iTCPRebindInterval, NULL));
-	CHKiRet(regCfSysLineHdlr((uchar *)"actionsendudprebindinterval", 0, eCmdHdlrInt, NULL, &cs.iUDPRebindInterval, NULL));
-	CHKiRet(regCfSysLineHdlr((uchar *)"actionsendtcpkeepalive", 0, eCmdHdlrBinary, NULL, &cs.bKeepAlive, NULL));
-	CHKiRet(regCfSysLineHdlr((uchar *)"actionsendtcpkeepalive_probes", 0, eCmdHdlrInt, NULL, &cs.iKeepAliveProbes, NULL));
-	CHKiRet(regCfSysLineHdlr((uchar *)"actionsendtcpkeepalive_intvl", 0, eCmdHdlrInt, NULL, &cs.iKeepAliveIntvl, NULL));
-	CHKiRet(regCfSysLineHdlr((uchar *)"actionsendtcpkeepalive_time", 0, eCmdHdlrInt, NULL, &cs.iKeepAliveTime, NULL));
-	CHKiRet(regCfSysLineHdlr((uchar *)"actionsendstreamdriver", 0, eCmdHdlrGetWord, NULL, &cs.pszStrmDrvr, NULL));
-	CHKiRet(regCfSysLineHdlr((uchar *)"actionsendstreamdrivermode", 0, eCmdHdlrInt, NULL, &cs.iStrmDrvrMode, NULL));
-	CHKiRet(regCfSysLineHdlr((uchar *)"actionsendstreamdriverauthmode", 0, eCmdHdlrGetWord, NULL, &cs.pszStrmDrvrAuthMode, NULL));
-	CHKiRet(regCfSysLineHdlr((uchar *)"actionsendstreamdriverpermittedpeer", 0, eCmdHdlrGetWord, setPermittedPeer, NULL, NULL));
-	CHKiRet(regCfSysLineHdlr((uchar *)"actionsendresendlastmsgonreconnect", 0, eCmdHdlrBinary, NULL, &cs.bResendLastOnRecon, NULL));
-	CHKiRet(omsdRegCFSLineHdlr((uchar *)"resetconfigvariables", 1, eCmdHdlrCustomHandler, resetConfigVariables, NULL, STD_LOADABLE_MODULE_ID));
+	CHKiRet(regCfSysLineHdlr((uchar *)"actionforwarddefaulttemplate", 0, eCmdHdlrGetWord,
+		setLegacyDfltTpl, NULL, NULL));
+	CHKiRet(regCfSysLineHdlr((uchar *)"actionsendtcprebindinterval", 0, eCmdHdlrInt,
+		NULL, &cs.iTCPRebindInterval, NULL));
+	CHKiRet(regCfSysLineHdlr((uchar *)"actionsendudprebindinterval", 0, eCmdHdlrInt,
+		NULL, &cs.iUDPRebindInterval, NULL));
+	CHKiRet(regCfSysLineHdlr((uchar *)"actionsendtcpkeepalive", 0, eCmdHdlrBinary,
+		NULL, &cs.bKeepAlive, NULL));
+	CHKiRet(regCfSysLineHdlr((uchar *)"actionsendtcpkeepalive_probes", 0, eCmdHdlrInt,
+		NULL, &cs.iKeepAliveProbes, NULL));
+	CHKiRet(regCfSysLineHdlr((uchar *)"actionsendtcpkeepalive_intvl", 0, eCmdHdlrInt,
+		NULL, &cs.iKeepAliveIntvl, NULL));
+	CHKiRet(regCfSysLineHdlr((uchar *)"actionsendtcpkeepalive_time", 0, eCmdHdlrInt,
+		NULL, &cs.iKeepAliveTime, NULL));
+	CHKiRet(regCfSysLineHdlr((uchar *)"actionsendstreamdriver", 0, eCmdHdlrGetWord,
+		NULL, &cs.pszStrmDrvr, NULL));
+	CHKiRet(regCfSysLineHdlr((uchar *)"actionsendstreamdrivermode", 0, eCmdHdlrInt,
+		NULL, &cs.iStrmDrvrMode, NULL));
+	CHKiRet(regCfSysLineHdlr((uchar *)"actionsendstreamdriverauthmode", 0, eCmdHdlrGetWord,
+		NULL, &cs.pszStrmDrvrAuthMode, NULL));
+	CHKiRet(regCfSysLineHdlr((uchar *)"actionsendstreamdriverpermittedpeer", 0, eCmdHdlrGetWord,
+		setPermittedPeer, NULL, NULL));
+	CHKiRet(regCfSysLineHdlr((uchar *)"actionsendresendlastmsgonreconnect", 0, eCmdHdlrBinary,
+		NULL, &cs.bResendLastOnRecon, NULL));
+	CHKiRet(omsdRegCFSLineHdlr((uchar *)"resetconfigvariables", 1, eCmdHdlrCustomHandler,
+		resetConfigVariables, NULL, STD_LOADABLE_MODULE_ID));
 ENDmodInit
 
 /* vim:set ai:
